from abjad import *
import py.test


def test_componenttools_get_nth_namesake_from_component_01():

    t = Staff("c'8 d'8 e'8 f'8")

    assert componenttools.get_nth_namesake_from_component(t[0], 0) is t[0]
    assert componenttools.get_nth_namesake_from_component(t[0], 1) is t[1]
    assert componenttools.get_nth_namesake_from_component(t[0], 2) is t[2]
    assert componenttools.get_nth_namesake_from_component(t[0], 3) is t[3]

    assert componenttools.get_nth_namesake_from_component(t[3], 0) is t[3]
    assert componenttools.get_nth_namesake_from_component(t[3], -1) is t[2]
    assert componenttools.get_nth_namesake_from_component(t[3], -2) is t[1]
    assert componenttools.get_nth_namesake_from_component(t[3], -3) is t[0]


def test_componenttools_get_nth_namesake_from_component_02():

    t = Staff("c'8 d'8 e'8 f'8")

    assert componenttools.get_nth_namesake_from_component(t[0], 99) is None


def test_componenttools_get_nth_namesake_from_component_03():
    '''Leaves within different anonymous parents have different
    parentage signatures and thus have no _next_namesake.
    '''

    t = Container(Voice(notetools.make_repeated_notes(2)) * 2)

    assert componenttools.get_nth_namesake_from_component(t.leaves[0], 1) is t.leaves[1]
    assert componenttools.get_nth_namesake_from_component(t.leaves[1], 1) is None
    assert componenttools.get_nth_namesake_from_component(t.leaves[2], 1) is t.leaves[3]


def test_componenttools_get_nth_namesake_from_component_04():
    '''Anonymous containers with the same parentage structure have
    different parentage signatures and thus have no _next_namesake.
    '''

    t = Container(Voice(notetools.make_repeated_notes(2)) * 2)

    assert componenttools.get_nth_namesake_from_component(t[0], 1) is None


def test_componenttools_get_nth_namesake_from_component_05():
    '''Differently named containers have a different parentage signature
    and thus do not _next_namesake.
    '''

    t = Container(Voice(notetools.make_repeated_notes(2)) * 2)
    t[0].name = 'voice'

    assert componenttools.get_nth_namesake_from_component(t[0], 1) is None
    assert componenttools.get_nth_namesake_from_component(t.leaves[0], 1) is t.leaves[1]
    assert componenttools.get_nth_namesake_from_component(t.leaves[1], 1) is None
    assert componenttools.get_nth_namesake_from_component(t.leaves[2], 1) is t.leaves[3]


def test_componenttools_get_nth_namesake_from_component_06():
    '''Calling _next_namesake on a named component when another component
    with the same type and name exists after the caller returns the first
    next namesake Component found.
    '''

    t = Container(Voice(notetools.make_repeated_notes(2)) * 2)
    t[0].name = 'voice'
    t[1].name = 'voice'

    assert componenttools.get_nth_namesake_from_component(t[0], 1) is t[1]
    assert componenttools.get_nth_namesake_from_component(t[1], 1) is None
    assert componenttools.get_nth_namesake_from_component(t.leaves[1], 1) is t.leaves[2]


def test_componenttools_get_nth_namesake_from_component_07():
    '''Components need not be strictly contiguous.
    '''

    t = Container(Voice(notetools.make_repeated_notes(2)) * 2)
    t[0].name = 'voice'
    t[1].name = 'voice'
    t.insert(1, Rest((1, 2)))

    assert componenttools.get_nth_namesake_from_component(t[0], 1) is t[2]
    assert componenttools.get_nth_namesake_from_component(t.leaves[1], 1) is t.leaves[3]


def test_componenttools_get_nth_namesake_from_component_08():
    '''Components need not thread (Staves don't thread).
    '''

    t = Container(Staff(notetools.make_repeated_notes(2)) * 2)
    t[0].name = 'staff'
    t[1].name = 'staff'

    assert componenttools.get_nth_namesake_from_component(t[0], 1) is t[1]
    assert componenttools.get_nth_namesake_from_component(t.leaves[1], 1) is t.leaves[2]


def test_componenttools_get_nth_namesake_from_component_09():
    '''_next_namesake works on parallel structures.
    '''

    a = Container(Voice(notetools.make_repeated_notes(2)) * 2)
    a[0].name = 'voiceOne'
    a[1].name = 'voiceTwo'
    a.is_parallel = True
    b = Container(Voice(notetools.make_repeated_notes(2)) * 2)
    b[0].name = 'voiceOne'
    b[1].name = 'voiceTwo'
    b.is_parallel = True
    t = Staff([a, b])

    r'''
    \new Staff {
        <<
            \context Voice = "voiceOne" {
                c'8
                c'8
            }
            \context Voice = "voiceTwo" {
                c'8
                c'8
            }
        >>
        <<
            \context Voice = "voiceOne" {
                c'8
                c'8
            }
            \context Voice = "voiceTwo" {
                c'8
                c'8
            }
        >>
    }
    '''

    assert componenttools.get_nth_namesake_from_component(a[0], 1) is b[0]
    assert componenttools.get_nth_namesake_from_component(a[1], 1) is b[1]
    assert componenttools.get_nth_namesake_from_component(a[0][1], 1) is b[0][0]
    assert componenttools.get_nth_namesake_from_component(a[1][1], 1) is b[1][0]



def list_written_durations_of_leaves_in_expr(expr):
    '''.. versionadded:: 2.0

    List the written durations of leaves in `expr`::

        >>> staff = Staff(tuplettools.FixedDurationTuplet(Duration(2, 8), "c'8 d'8 e'8") * 2)

    ::

        >>> for duration in leaftools.list_written_durations_of_leaves_in_expr(staff):
        ...     duration
        ...
        Duration(1, 8)
        Duration(1, 8)
        Duration(1, 8)
        Duration(1, 8)
        Duration(1, 8)    
         Duration(1, 8)

    Return list of durations.
    '''
    from abjad.tools import iterationtools

    durations = []

    for leaf in iterationtools.iterate_leaves_in_expr(expr):
        durations.append(leaf.written_duration)

    return durations



import fractions


def numeric_seconds_to_clock_string(seconds, escape_ticks=False):
    r'''.. versionadded:: 2.0

    Example 1. Change numeric `seconds` to clock string::

        >>> durationtools.numeric_seconds_to_clock_string(117)
        '1\'57"'

    Example 2. Change numeric `seconds` to escaped clock string::

        >>> note = Note("c'4")
        >>> clock_string = durationtools.numeric_seconds_to_clock_string(117, escape_ticks=True)

    ::

        >>> markuptools.Markup('"%s"' % clock_string, Up)(note)
        Markup(('1\'57\\"',), direction=Up)(c'4)

    ::

        >>> f(note)
        c'4 ^ \markup { 1'57\\" }

    Return string.
    '''

    assert isinstance(seconds, (int, float, fractions.Fraction))
    if seconds < 0:
        raise ValueError('total seconds must be positive.')

    minutes = int(seconds / 60)
    remaining_seconds = str(int(seconds - minutes * 60)).zfill(2)
    if escape_ticks:
        clock_string = "%s'%s\\\"" % (minutes, remaining_seconds)
    else:
        clock_string = "%s'%s\"" % (minutes, remaining_seconds)

    return clock_string



from abjad.tools import componenttools


def all_are_measures(expr):
    '''.. versionadded:: 2.6

    True when `expr` is a sequence of Abjad measures::

        >>> measures = 3 * Measure((3, 4), "c'4 d'4 e'4")

    ::

        >>> for measure in measures:
        ...     measure
        ...
        Measure(3/4, [c'4, d'4, e'4])
        Measure(3/4, [c'4, d'4, e'4])
        Measure(3/4, [c'4, d'4, e'4])

    ::

        >>> measuretools.all_are_measures(measures)
        True

    True when `expr` is an empty sequence::

        >>> measuretools.all_are_measures([])
        True

    Otherwise false::

        >>> measuretools.all_are_measures('foo')
        False

    Return boolean.

    Function wraps ``componenttools.all_are_components()``.
    '''
    from abjad.tools import measuretools

    return componenttools.all_are_components(expr, klasses=(measuretools.Measure,))



def list_package_dependency_versions():
    r'''List package dependency versions::

        >>> configurationtools.list_package_dependency_versions() # doctest: +SKIP
        {'sphinx': '1.1.2', 'py.test': '2.1.2'}

    Return dictionary.  
    '''

    deps = {}

    deps['configobj'] = None
    try:
        import configobj
        deps['configobj'] = configobj.__version__
    except ImportError:
        pass

    deps['ply'] = None
    try:
        from ply import lex
        deps['ply'] = lex.__version__
    except ImportError:
        pass

    # py.test
    deps['py.test'] = None
    try:
        import py.test
        deps['py.test'] = py.test.__version__
    except ImportError:
        pass

    # sphinx
    deps['sphinx'] = None
    try:
        import sphinx
        deps['sphinx'] = sphinx.__version__
    except ImportError:
        pass

    return deps



from abjad.tools import durationtools
from abjad.tools import quantizationtools


def test_TerminalQEvent___init___01():

    q_event = quantizationtools.TerminalQEvent(154)

    assert q_event.offset == durationtools.Offset(154)



'''Abjad ``scoretools`` package.

    Dependencies:

    The ``scoretools`` package may import the ``containertools``
    and ``componenttool`` packages at top level.

    The ``scoretools`` package should not import sibling
    or cousing packages like ``stafftools`` at top level.

    The ``scoretools`` package should not import ``instrumenttools`` 
    at top level.
    
'''
from abjad.tools import importtools

importtools.import_structured_package(__path__[0], globals(), package_root_name='abjad')

_documentation_section = 'core'



from abjad import *


def test_NoteRhythmMaker_new_01():

    maker = rhythmmakertools.NoteRhythmMaker()

    divisions = [(5, 16), (3, 8)]
    leaf_lists = maker.new(decrease_durations_monotonically=False)(divisions)
    leaves = sequencetools.flatten_sequence(leaf_lists)

    staff = Staff(measuretools.make_measures_with_full_measure_spacer_skips(divisions))
    measuretools.replace_contents_of_measures_in_expr(staff, leaves)

    r'''
    \new Staff {
        {
            \time 5/16
            c'16 ~
            c'4
        }
        {
            \time 3/8
            c'4.
        }
    }
    '''

    assert staff.lilypond_format == "\\new Staff {\n\t{\n\t\t\\time 5/16\n\t\tc'16 ~\n\t\tc'4\n\t}\n\t{\n\t\t\\time 3/8\n\t\tc'4.\n\t}\n}"



from abjad import *


def test_marktools_move_marks_01():

    staff = Staff(r'\clef "bass" c \staccato d e f')
        
    assert len(marktools.get_marks_attached_to_component(staff[0])) == 2
    assert len(marktools.get_marks_attached_to_component(staff[1])) == 0
    assert len(marktools.get_marks_attached_to_component(staff[2])) == 0
    assert len(marktools.get_marks_attached_to_component(staff[3])) == 0

    marktools.move_marks(staff[0], staff[2])

    assert len(marktools.get_marks_attached_to_component(staff[0])) == 0
    assert len(marktools.get_marks_attached_to_component(staff[1])) == 0
    assert len(marktools.get_marks_attached_to_component(staff[2])) == 2
    assert len(marktools.get_marks_attached_to_component(staff[3])) == 0



from abjad import *


def test_Markup___repr___01():

    markup = markuptools.Markup('foo bar')
    assert repr(markup) == "Markup(('foo bar',))"


def test_Markup___repr___02():

    markup = markuptools.Markup('foo bar', direction=Up)
    assert repr(markup) == "Markup(('foo bar',), direction=Up)"


def test_Markup___repr___03():

    markup = markuptools.Markup('foo bar', direction=Up, markup_name='foo')
    assert repr(markup) == "Markup(('foo bar',), direction=Up, markup_name='foo')"


def test_Markup___repr___04():

    markup = markuptools.Markup(('foo', 'bar'), direction=Up, markup_name='foo')
    assert repr(markup) == "Markup(('foo', 'bar'), direction=Up, markup_name='foo')"



from abjad.tools import *


def test_OrdinalConstant_storage_format_01():
    '''Storage format exists and is evaluable.
    '''

    vector_constant_1 = datastructuretools.OrdinalConstant('x', -1, 'Left')
    storage_format = vector_constant_1.storage_format

    assert storage_format == 'Left'
    vector_constant_2 = eval(storage_format)

    assert isinstance(vector_constant_1, datastructuretools.OrdinalConstant)
    assert isinstance(vector_constant_2, datastructuretools.OrdinalConstant)
    assert not vector_constant_1 is vector_constant_2 
    assert vector_constant_1 == vector_constant_2



import math
from abjad.tools import containertools
from abjad.tools import durationtools
from abjad.tools import leaftools
from abjad.tools import mathtools
from abjad.tools import notetools
from abjad.tools import resttools


# TODO: change name to make_tuplet_from_nonreduced_ratio_and_nonreduced_fraction
def make_tuplet_from_nonreduced_ratio_and_nonreduced_fraction(proportions, (n, d)):
    '''Divide nonreduced fraction `(n, d)` according to `proportions`.

    Return container when no prolation is necessary::

        >>> tuplettools.make_tuplet_from_nonreduced_ratio_and_nonreduced_fraction(
        ...     [1], (7, 16))
        {c'4..}

    Return fixed-duration tuplet when prolation is necessary::

        >>> tuplettools.make_tuplet_from_nonreduced_ratio_and_nonreduced_fraction(
        ...     [1, 2], (7, 16))
        FixedDurationTuplet(7/16, [c'8, c'4])

    ::

        >>> tuplettools.make_tuplet_from_nonreduced_ratio_and_nonreduced_fraction(
        ...     [1, 2, 4], (7, 16))
        FixedDurationTuplet(7/16, [c'16, c'8, c'4])

    ::

        >>> tuplettools.make_tuplet_from_nonreduced_ratio_and_nonreduced_fraction(
        ...     [1, 2, 4, 1], (7, 16))
        FixedDurationTuplet(7/16, [c'16, c'8, c'4, c'16])

    ::

        >>> tuplettools.make_tuplet_from_nonreduced_ratio_and_nonreduced_fraction(
        ...     [1, 2, 4, 1, 2], (7, 16))
        FixedDurationTuplet(7/16, [c'16, c'8, c'4, c'16, c'8])

    ::

        >>> tuplettools.make_tuplet_from_nonreduced_ratio_and_nonreduced_fraction(
        ...     [1, 2, 4, 1, 2, 4], (7, 16))
        FixedDurationTuplet(7/16, [c'16, c'8, c'4, c'16, c'8, c'4])

    .. note:: function interprets `d` as tuplet denominator.

    Return tuplet or container.

    .. versionchanged:: 2.0
        renamed ``divide.pair()`` to
        ``tuplettools.make_tuplet_from_nonreduced_ratio_and_nonreduced_fraction()``.
    '''
    from abjad.tools import tuplettools

    # check input
    proportions = mathtools.NonreducedRatio(proportions)
    duration = durationtools.Duration(n, d)
    
    if len(proportions) == 1:
        if 0 < proportions[0]:
            try:
                return containertools.Container([notetools.Note(0, duration)])
            except AssignabilityError:
                return containertools.Container(notetools.make_notes(0, duration))
        elif proportions[0] < 0:
            try:
                return containertools.Container([resttools.Rest(duration)])
            except AssignabilityError:
                return containertools.Container(resttools.make_rests(duration))
        else:
            raise ValueError('no divide zero values.')

    if 1 < len(proportions):
        exponent = int(math.log(mathtools.weight(proportions), 2) - math.log(n, 2))
        denominator = int(d * 2 ** exponent)
        music = []
        for x in proportions:
            if not x:
                raise ValueError('no divide zero values.')
            if 0 < x:
                try:
                    music.append(notetools.Note(0, (x, denominator)))
                except AssignabilityError:
                    music.extend(notetools.make_notes(0, (x, denominator)))
            else:
                music.append(resttools.Rest((-x, denominator)))
        return tuplettools.FixedDurationTuplet(duration, music)



from abjad.tools.lilypondfiletools.AttributedBlock import AttributedBlock
from abjad.tools import lilypondproxytools


class ContextBlock(AttributedBlock):
    r'''.. versionadded:: 2.5

    Abjad model of LilyPond input file context block::

        >>> context_block = lilypondfiletools.ContextBlock()

    ::

        >>> context_block
        ContextBlock()

    ::

        >>> context_block.context_name = 'Score'
        >>> context_block.override.bar_number.transparent = True
        >>> scheme = schemetools.Scheme('end-of-line-invisible')
        >>> context_block.override.time_signature.break_visibility = scheme
        >>> context_block.set.proportionalNotationDuration = schemetools.SchemeMoment((1, 45))

    ::

        >>> f(context_block)
        \context {
            \Score
            \override BarNumber #'transparent = ##t
            \override TimeSignature #'break-visibility = #end-of-line-invisible
            proportionalNotationDuration = #(ly:make-moment 1 45)
        }

    Return context block.
    '''

    ### INITIALIZER ###

    def __init__(self, context_name=None):
        AttributedBlock.__init__(self)
        self._accepts = []
        self._engraver_consists = []
        self._engraver_removals = []
        self._escaped_name = r'\context'
        self.context_name = context_name
        self.name = None
        self.type = None
        
    ### PRIVATE PROPERTIES ###

    @property
    def _format_pieces(self):
        from abjad.tools.lilypondfiletools._format_lilypond_context_setting_in_with_block import \
            _format_lilypond_context_setting_in_with_block
        result = []
        result.append('%s {' % self._escaped_name)
        if self.type is not None:
            result.append('\t' + r'\type %s' % self.type)
        if self.context_name is not None:
            result.append('\t' + r'\%s' % self.context_name)
        if self.name is not None:
            result.append('\t' + r'\name %s' % self.name)
        for string in self.engraver_removals:
            result.append('\t' + r'\remove %s' % string)
        # CAUTION: LilyPond consist statements are order-significant!
        for string in self.engraver_consists:
            result.append('\t' + r'\consists %s' % string)
        for string in self.accepts:
            result.append('\t' + r'\accepts %s' % string)
        for override in self.override._list_format_contributions('override'):
            result.append('\t' + override)
        setting_contributions = []
        for key, value in self.set._get_attribute_tuples():
            setting_contribution = _format_lilypond_context_setting_in_with_block(key, value)
            setting_contributions.append(setting_contribution)
        for setting_contribution in sorted(setting_contributions):
            result.append('\t' + setting_contribution)
        result.append('}')
        return result

    ### READ-ONLY PUBLIC PROPERTIES ###

    @property
    def accepts(self):
        return self._accepts

    @property
    def engraver_consists(self):
        return self._engraver_consists

    @property
    def engraver_removals(self):
        return self._engraver_removals

    @property
    def override(self):
        '''Read-only reference to LilyPond grob override component plug-in.
        '''
        if not hasattr(self, '_override'):
            self._override = lilypondproxytools.LilyPondGrobOverrideComponentPlugIn()
        return self._override

    @property
    def set(self):
        '''Read-only reference LilyPond context setting component plug-in.
        '''
        if not hasattr(self, '_set'):
            self._set = lilypondproxytools.LilyPondContextSettingComponentPlugIn()
        return self._set

    ### READ / WRITE PUBLIC PROPERTIES ###

    @apply
    def context_name():
        def fget(self):
            r'''Read / write context name.'''
            return self._context_name
        def fset(self, context_name):
            assert isinstance(context_name, (str, type(None)))
            self._context_name = context_name
        return property(**locals())

    @apply
    def name():
        def fget(self):
            r'''Read / write name.'''
            return self._name
        def fset(self, name):
            assert isinstance(name, (str, type(None)))
            self._name = name
        return property(**locals())

    @apply
    def type():
        def fget(self):
            r'''Read / write type.'''
            return self._type
        def fset(self, expr):
            assert isinstance(expr, (str, type(None)))
            self._type = expr
        return property(**locals())



from abjad.tools import abctools
from abjad.tools import mathtools
from abjad.tools.durationtools.Duration import Duration


class Multiplier(Duration):
    '''.. versionadded:: 2.11

    Multiplier.
    '''

    ### SPECIAL METHODS ###

    # multiplier times duration gives duration
    def __mul__(self, *args):
        if len(args) == 1 and args[0].__class__.__name__ == 'Duration':
            return Duration(Duration.__mul__(self, *args))
        else:
            return Duration.__mul__(self, *args)

    ### PUBLIC PROPERTIES ###

    @property
    def is_proper_tuplet_multiplier(self):
        '''.. versionadded:: 2.11

        True when mutliplier is greater than ``1/2`` and less than ``2``.
        Otherwise false::

            >>> Multiplier(3, 2).is_proper_tuplet_multiplier
            True

        Return boolean.
        '''
        return type(self)(1, 2) < self < type(self)(2)



from abjad.tools import iterationtools
from abjad.tools import marktools


def apply_final_bar_lines(score):

    for voice in iterationtools.iterate_voices_in_expr(score):
        marktools.BarLine('|.')(voice[-1])



from abjad import *


def test_containertools_delete_contents_of_container_starting_at_or_after_offset_01():

    staff = Staff("c'8 d'8 e'8 f'8")
    beamtools.BeamSpanner(staff.leaves)
    containertools.delete_contents_of_container_starting_at_or_after_offset(staff, Duration(1, 8))

    r'''
    \new Staff {
        c'8 [ ]
    }
    '''

    assert wellformednesstools.is_well_formed_component(staff)
    assert staff.lilypond_format == "\\new Staff {\n\tc'8 [ ]\n}"


def test_containertools_delete_contents_of_container_starting_at_or_after_offset_02():

    staff = Staff("c'8 d'8 e'8 f'8")
    beamtools.BeamSpanner(staff.leaves)
    containertools.delete_contents_of_container_starting_at_or_after_offset(staff, Duration(3, 16))

    r'''
    \new Staff {
        c'8 [
        d'8 ]
    }
    '''

    assert wellformednesstools.is_well_formed_component(staff)
    assert staff.lilypond_format == "\\new Staff {\n\tc'8 [\n\td'8 ]\n}"



from abjad.tools import configurationtools
import os


def test_AbjadConfig___init___01():

    ABJADCONFIG = configurationtools.AbjadConfig()

    assert os.path.exists(ABJADCONFIG.ABJAD_CONFIG_DIRECTORY_PATH)
    assert os.path.exists(ABJADCONFIG.ABJAD_CONFIG_FILE_PATH)
    assert os.path.exists(ABJADCONFIG.ABJAD_OUTPUT_PATH)

    keys = [
        'text_editor',
        'pdf_viewer',
        'lilypond_includes',
        'lilypond_path',
        'midi_player',
        'abjad_output',
        'accidental_spelling',
        'lilypond_language',
    ]

    for key in keys:
        assert key in ABJADCONFIG



from abjad import *
import py


def test_containertools_split_container_at_index_01():
    '''Split tuplet in score and do not fracture spanners.
    '''

    t = Voice(tuplettools.FixedDurationTuplet(Duration(2, 8), notetools.make_repeated_notes(3)) * 2)
    pitchtools.set_ascending_named_diatonic_pitches_on_tie_chains_in_expr(t)
    p = beamtools.BeamSpanner(t[:])

    r'''
    \new Voice {
        \times 2/3 {
            c'8
            d'8
            e'8
        }
        \times 2/3 {
            f'8
            g'8
            a'8
        }
    }
    '''

    containertools.split_container_at_index(t[1], 1, fracture_spanners=False)

    r'''
    \new Voice {
        \times 2/3 {
            c'8 [
            d'8
            e'8
        }
        \times 2/3 {
            f'8
        }
        \times 2/3 {
            g'8
            a'8 ]
        }
    }
    '''

    assert wellformednesstools.is_well_formed_component(t)
    assert t.lilypond_format == "\\new Voice {\n\t\\times 2/3 {\n\t\tc'8 [\n\t\td'8\n\t\te'8\n\t}\n\t\\times 2/3 {\n\t\tf'8\n\t}\n\t\\times 2/3 {\n\t\tg'8\n\t\ta'8 ]\n\t}\n}"


def test_containertools_split_container_at_index_02():
    '''Split in-score measure with power-of-two denominator and do not fracture spanners.
    '''

    t = Voice(Measure((3, 8), notetools.make_repeated_notes(3)) * 2)
    pitchtools.set_ascending_named_diatonic_pitches_on_tie_chains_in_expr(t)
    p = beamtools.BeamSpanner(t[:])

    r'''
    \new Voice {
        {
            \time 3/8
            c'8 [
            d'8
            e'8
        }
        {
            \time 3/8
            f'8
            g'8
            a'8 ]
        }
    }
    '''

    containertools.split_container_at_index(t[1], 1, fracture_spanners=False)

    r'''
    \new Voice {
        {
            \time 3/8
            c'8 [
            d'8
            e'8
        }
        {
            \time 1/8
            f'8
        }
        {
            \time 2/8
            g'8
            a'8 ]
        }
    }
    '''

    assert wellformednesstools.is_well_formed_component(t)
    assert t.lilypond_format == "\\new Voice {\n\t{\n\t\t\\time 3/8\n\t\tc'8 [\n\t\td'8\n\t\te'8\n\t}\n\t{\n\t\t\\time 1/8\n\t\tf'8\n\t}\n\t{\n\t\t\\time 2/8\n\t\tg'8\n\t\ta'8 ]\n\t}\n}"



def test_containertools_split_container_at_index_03():
    '''Split in-score measure without power-of-two denominator and do not frature spanners.
    '''

    t = Voice(Measure((3, 9), notetools.make_repeated_notes(3)) * 2)
    pitchtools.set_ascending_named_diatonic_pitches_on_tie_chains_in_expr(t)
    p = beamtools.BeamSpanner(t[:])

    r'''
    \new Voice {
        {
            \time 3/9
            \scaleDurations #'(8 . 9) {
                c'8 [
                d'8
                e'8
            }
        }
        {
            \time 3/9
            \scaleDurations #'(8 . 9) {
                f'8
                g'8
                a'8 ]
            }
        }
    }
    '''

    containertools.split_container_at_index(t[1], 1, fracture_spanners=False)

    r'''
    \new Voice {
        {
            \time 3/9
            \scaleDurations #'(8 . 9) {
                c'8 [
                d'8
                e'8
            }
        }
        {
            \time 1/9
            \scaleDurations #'(8 . 9) {
                f'8
            }
        }
        {
            \time 2/9
            \scaleDurations #'(8 . 9) {
                g'8
                a'8 ]
            }
        }
    }
    '''

    assert wellformednesstools.is_well_formed_component(t)
    assert t.lilypond_format == "\\new Voice {\n\t{\n\t\t\\time 3/9\n\t\t\\scaleDurations #'(8 . 9) {\n\t\t\tc'8 [\n\t\t\td'8\n\t\t\te'8\n\t\t}\n\t}\n\t{\n\t\t\\time 1/9\n\t\t\\scaleDurations #'(8 . 9) {\n\t\t\tf'8\n\t\t}\n\t}\n\t{\n\t\t\\time 2/9\n\t\t\\scaleDurations #'(8 . 9) {\n\t\t\tg'8\n\t\t\ta'8 ]\n\t\t}\n\t}\n}"
    #assert t.lilypond_format == "\\new Voice {\n\t{\n\t\t\\scaleDurations #'(8 . 9) {\n\t\t\t\\time 3/9\n\t\t\tc'8 [\n\t\t\td'8\n\t\t\te'8\n\t\t}\n\t}\n\t{\n\t\t\\scaleDurations #'(8 . 9) {\n\t\t\t\\time 1/9\n\t\t\tf'8\n\t\t}\n\t}\n\t{\n\t\t\\scaleDurations #'(8 . 9) {\n\t\t\t\\time 2/9\n\t\t\tg'8\n\t\t\ta'8 ]\n\t\t}\n\t}\n}"


def test_containertools_split_container_at_index_04():
    '''A single container can be index split in two by the middle; no parent.
    '''

    t = Voice("c'8 d'8 e'8 f'8")
    t1, t2 = containertools.split_container_at_index(t, 2, fracture_spanners=False)

    r'''
    \new Voice {
        c'8
        d'8
    }
    \new Voice {
        e'8
        f'8
    }
    '''

    assert wellformednesstools.is_well_formed_component(t1)
    assert wellformednesstools.is_well_formed_component(t2)
    assert t1.lilypond_format == "\\new Voice {\n\tc'8\n\td'8\n}"
    assert t2.lilypond_format == "\\new Voice {\n\te'8\n\tf'8\n}"


def test_containertools_split_container_at_index_05():
    '''A single container 'split' at index 0 gives
    an empty lefthand part and a complete righthand part.
    Original container empties contents.
    '''

    t = Staff([Voice("c'8 d'8 e'8 f'8")])
    v = t[0]
    beamtools.BeamSpanner(v)
    left, right = containertools.split_container_at_index(v, 0, fracture_spanners=False)

    r'''
    \new Staff {
        \new Voice {
            c'8 [
            d'8
            e'8
            f'8 ]
        }
    }
    '''

    assert wellformednesstools.is_well_formed_component(t)
    assert left.lilypond_format == '\\new Voice {\n}'
    assert right.lilypond_format == "\\new Voice {\n\tc'8 [\n\td'8\n\te'8\n\tf'8 ]\n}"
    assert t.lilypond_format == "\\new Staff {\n\t\\new Voice {\n\t\tc'8 [\n\t\td'8\n\t\te'8\n\t\tf'8 ]\n\t}\n}"


def test_containertools_split_container_at_index_06():
    '''Split container at index greater than len(container).
    Lefthand part instantiates with all contents.
    Righthand part instantiates empty.
    Original container empties contents.
    '''

    t = Staff([Voice("c'8 d'8 e'8 f'8")])
    v = t[0]
    left, right = containertools.split_container_at_index(v, 10, fracture_spanners=False)

    assert wellformednesstools.is_well_formed_component(t)
    assert left.lilypond_format == "\\new Voice {\n\tc'8\n\td'8\n\te'8\n\tf'8\n}"
    assert right.lilypond_format == '\\new Voice {\n}'
    assert v.lilypond_format == '\\new Voice {\n}'
    assert t.lilypond_format == "\\new Staff {\n\t\\new Voice {\n\t\tc'8\n\t\td'8\n\t\te'8\n\t\tf'8\n\t}\n}"


def test_containertools_split_container_at_index_07():
    '''Voice can be index split.
    '''

    t = Staff([Voice("c'8 d'8 e'8 f'8")])
    v = t[0]
    #assert py.test.raises(ContiguityError, 'containertools.split_container_at_index(v, -2, fracture_spanners=False)')

    left, right = containertools.split_container_at_index(v, -2, fracture_spanners=False)
    assert wellformednesstools.is_well_formed_component(t)
    assert left.lilypond_format == "\\new Voice {\n\tc'8\n\td'8\n}"
    assert right.lilypond_format == "\\new Voice {\n\te'8\n\tf'8\n}"
    assert v.lilypond_format == '\\new Voice {\n}'
    assert t.lilypond_format == "\\new Staff {\n\t\\new Voice {\n\t\tc'8\n\t\td'8\n\t}\n\t\\new Voice {\n\t\te'8\n\t\tf'8\n\t}\n}"


def test_containertools_split_container_at_index_08():
    '''Slit container in score and do not fracture spanners.
    '''

    t = Staff([Container("c'8 d'8 e'8 f'8")])
    v = t[0]
    beamtools.BeamSpanner(v)
    left, right = containertools.split_container_at_index(v, 2, fracture_spanners=False)

    r'''
    \new Staff {
        {
            c'8 [
            d'8
        }
        {
            e'8
            f'8 ]
        }
    }
    '''

    assert wellformednesstools.is_well_formed_component(t)
    assert left.lilypond_format == "{\n\tc'8 [\n\td'8\n}"
    assert right.lilypond_format == "{\n\te'8\n\tf'8 ]\n}"
    assert v.lilypond_format == '{\n}'
    assert t.lilypond_format == "\\new Staff {\n\t{\n\t\tc'8 [\n\t\td'8\n\t}\n\t{\n\t\te'8\n\t\tf'8 ]\n\t}\n}"


def test_containertools_split_container_at_index_09():
    '''Split tuplet in score and do not fracture spanners.
    '''

    t = Staff([Voice([Tuplet(Fraction(4, 5), notetools.make_repeated_notes(5))])])
    v = t[0]
    tuplet = v[0]
    beamtools.BeamSpanner(tuplet)
    left, right = containertools.split_container_at_index(tuplet, 2, fracture_spanners=False)

    r'''
    \new Staff {
        \new Voice {
            \times 4/5 {
                c'8 [
                c'8
            }
            \times 4/5 {
                c'8
                c'8
                c'8 ]
            }
        }
    }
    '''

    assert wellformednesstools.is_well_formed_component(t)
    assert left.lilypond_format == "\\times 4/5 {\n\tc'8 [\n\tc'8\n}"
    assert right.lilypond_format == "\\times 4/5 {\n\tc'8\n\tc'8\n\tc'8 ]\n}"
    assert tuplet.lilypond_format == '\\times 4/5 {\n}'
    assert v.lilypond_format == "\\new Voice {\n\t\\times 4/5 {\n\t\tc'8 [\n\t\tc'8\n\t}\n\t\\times 4/5 {\n\t\tc'8\n\t\tc'8\n\t\tc'8 ]\n\t}\n}"
    assert t.lilypond_format == "\\new Staff {\n\t\\new Voice {\n\t\t\\times 4/5 {\n\t\t\tc'8 [\n\t\t\tc'8\n\t\t}\n\t\t\\times 4/5 {\n\t\t\tc'8\n\t\t\tc'8\n\t\t\tc'8 ]\n\t\t}\n\t}\n}"


def test_containertools_split_container_at_index_10():
    '''Split left of leaf in score and do not fracture spanners.
    '''

    t = Staff(Measure((2, 8), notetools.make_repeated_notes(2)) * 2)
    pitchtools.set_ascending_named_diatonic_pitches_on_tie_chains_in_expr(t)
    beamtools.BeamSpanner(t[0])
    beamtools.BeamSpanner(t[1])
    slur = spannertools.SlurSpanner(t.leaves)
    measuretools.set_always_format_time_signature_of_measures_in_expr(t)

    r'''
    \new Staff {
        {
            \time 2/8
            c'8 [ (
            d'8 ]
        }
        {
            \time 2/8
            e'8 [
            f'8 ] )
        }
    }
    '''

    leaf = t.leaves[1]
    left, right = containertools.split_container_at_index(leaf, -100, fracture_spanners=False)

    "Score is unchanged."

    r'''
    \new Staff {
        {
            \time 2/8
            c'8 [ (
            d'8 ]
        }
        {
            \time 2/8
            e'8 [
            f'8 ] )
        }
    }
    '''

    assert wellformednesstools.is_well_formed_component(t)
    assert left is None
    assert right is leaf
    assert t.lilypond_format == "\\new Staff {\n\t{\n\t\t\\time 2/8\n\t\tc'8 [ (\n\t\td'8 ]\n\t}\n\t{\n\t\t\\time 2/8\n\t\te'8 [\n\t\tf'8 ] )\n\t}\n}"


def test_containertools_split_container_at_index_11():
    '''Split right of leaf in score and do not fracture spanners.
    '''

    t = Staff(Measure((2, 8), notetools.make_repeated_notes(2)) * 2)
    pitchtools.set_ascending_named_diatonic_pitches_on_tie_chains_in_expr(t)
    beamtools.BeamSpanner(t[0])
    beamtools.BeamSpanner(t[1])
    slur = spannertools.SlurSpanner(t.leaves)
    measuretools.set_always_format_time_signature_of_measures_in_expr(t)

    r'''
    \new Staff {
        {
            \time 2/8
            c'8 [ (
            d'8 ]
        }
        {
            \time 2/8
            e'8 [
            f'8 ] )
        }
    }
    '''

    leaf = t.leaves[1]
    left, right = containertools.split_container_at_index(leaf, 100, fracture_spanners=False)

    "Score is unchanged."

    r'''
    \new Staff {
        {
            \time 2/8
            c'8 [ (
            d'8 ]
        }
        {
            \time 2/8
            e'8 [
            f'8 ] )
        }
    }
    '''

    assert wellformednesstools.is_well_formed_component(t)
    assert left is leaf
    assert right is None
    assert t.lilypond_format == "\\new Staff {\n\t{\n\t\t\\time 2/8\n\t\tc'8 [ (\n\t\td'8 ]\n\t}\n\t{\n\t\t\\time 2/8\n\t\te'8 [\n\t\tf'8 ] )\n\t}\n}"


def test_containertools_split_container_at_index_12():
    '''Split triplet, and fracture spanners.
    '''

    t = Voice(tuplettools.FixedDurationTuplet(Duration(2, 8), notetools.make_repeated_notes(3)) * 2)
    tuplet = t[1]
    pitchtools.set_ascending_named_diatonic_pitches_on_tie_chains_in_expr(t)
    beamtools.BeamSpanner(t[:])

    r'''
    \new Voice {
        \times 2/3 {
            c'8 [
            d'8
            e'8
        }
        \times 2/3 {
            f'8
            g'8
            a'8 ]
        }
    }
    '''

    left, right = containertools.split_container_at_index(tuplet, 1, fracture_spanners=True)

    r'''
    \new Voice {
        \times 2/3 {
            c'8 [
            d'8
            e'8
        }
        \times 2/3 {
            f'8 ]
        }
        \times 2/3 {
            g'8 [
            a'8 ]
        }
    }
    '''

    assert wellformednesstools.is_well_formed_component(t)
    assert left.lilypond_format == "\\times 2/3 {\n\tf'8 ]\n}"
    assert right.lilypond_format == "\\times 2/3 {\n\tg'8 [\n\ta'8 ]\n}"
    assert tuplet.lilypond_format == ''
    assert t.lilypond_format == "\\new Voice {\n\t\\times 2/3 {\n\t\tc'8 [\n\t\td'8\n\t\te'8\n\t}\n\t\\times 2/3 {\n\t\tf'8 ]\n\t}\n\t\\times 2/3 {\n\t\tg'8 [\n\t\ta'8 ]\n\t}\n}"


def test_containertools_split_container_at_index_13():
    '''Split measure with power-of-two time signature denominator.
    Fracture spanners.
    '''

    t = Voice(Measure((3, 8), notetools.make_repeated_notes(3)) * 2)
    m = t[1]
    beamtools.BeamSpanner(t[:])
    pitchtools.set_ascending_named_diatonic_pitches_on_tie_chains_in_expr(t)

    r'''
    \new Voice {
        {
            \time 3/8
            c'8 [
            d'8
            e'8
        }
        {
            \time 3/8
            f'8
            g'8
            a'8 ]
        }
    }
    '''

    left, right = containertools.split_container_at_index(m, 1, fracture_spanners=True)

    r'''
    \new Voice {
        {
            \time 3/8
            c'8 [
            d'8
            e'8
        }
        {
            \time 1/8
            f'8 ]
        }
        {
            \time 2/8
            g'8 [
            a'8 ]
        }
    }
    '''

    assert wellformednesstools.is_well_formed_component(t)
    assert left.lilypond_format == "{\n\t\\time 1/8\n\tf'8 ]\n}"
    assert right.lilypond_format == "{\n\t\\time 2/8\n\tg'8 [\n\ta'8 ]\n}"
    assert py.test.raises(UnderfullContainerError, 'm.lilypond_format')
    assert t.lilypond_format == "\\new Voice {\n\t{\n\t\t\\time 3/8\n\t\tc'8 [\n\t\td'8\n\t\te'8\n\t}\n\t{\n\t\t\\time 1/8\n\t\tf'8 ]\n\t}\n\t{\n\t\t\\time 2/8\n\t\tg'8 [\n\t\ta'8 ]\n\t}\n}"


def test_containertools_split_container_at_index_14():
    '''Split measure without power-of-two time signature denominator.
    Fracture spanners.
    '''

    t = Voice(Measure((3, 9), notetools.make_repeated_notes(3)) * 2)
    m = t[1]
    beamtools.BeamSpanner(t[:])
    pitchtools.set_ascending_named_diatonic_pitches_on_tie_chains_in_expr(t)

    r'''
    \new Voice {
        {
            \time 3/9
            \scaleDurations '(8 . 9) {
                c'8 [
                d'8
                e'8
            }
        }
        {
            \time 3/9
            \scaleDurations '(8 . 9) {
                f'8
                g'8
                a'8 ]
            }
        }
    }
    '''

    left, right = containertools.split_container_at_index(m, 1, fracture_spanners=True)

    r'''
    \new Voice {
        {
            \time 3/9
            \scaleDurations '(8 . 9) {
                c'8 [
                d'8
                e'8
            }
        }
        {
            \time 1/9
            \scaleDurations '(8 . 9) {
                f'8 ]
            }
        }
        {
            \time 2/9
            \scaleDurations '(8 . 9) {
                g'8 [
                a'8 ]
            }
        }
    }
    '''

    assert wellformednesstools.is_well_formed_component(t)
    assert left.lilypond_format == "{\n\t\\time 1/9\n\t\\scaleDurations #'(8 . 9) {\n\t\tf'8 ]\n\t}\n}"
    assert right.lilypond_format == "{\n\t\\time 2/9\n\t\\scaleDurations #'(8 . 9) {\n\t\tg'8 [\n\t\ta'8 ]\n\t}\n}"
    assert py.test.raises(UnderfullContainerError, 'm.lilypond_format')
    assert t.lilypond_format == "\\new Voice {\n\t{\n\t\t\\time 3/9\n\t\t\\scaleDurations #'(8 . 9) {\n\t\t\tc'8 [\n\t\t\td'8\n\t\t\te'8\n\t\t}\n\t}\n\t{\n\t\t\\time 1/9\n\t\t\\scaleDurations #'(8 . 9) {\n\t\t\tf'8 ]\n\t\t}\n\t}\n\t{\n\t\t\\time 2/9\n\t\t\\scaleDurations #'(8 . 9) {\n\t\t\tg'8 [\n\t\t\ta'8 ]\n\t\t}\n\t}\n}"


def test_containertools_split_container_at_index_15():
    '''Split voice outside of score.
    Fracture spanners.
    '''

    t = Voice("c'8 d'8 e'8 f'8")
    beamtools.BeamSpanner(t[:])

    r'''
    \new Voice {
        c'8 [
        d'8
        e'8
        f'8 ]
    }
    '''

    left, right = containertools.split_container_at_index(t, 2, fracture_spanners=True)

    r'''
    \new Voice {
        c'8 [
        d'8
    }
    '''

    r'''
    \new Voice {
        e'8
        f'8 ]
    }
    '''

    assert left.lilypond_format == "\\new Voice {\n\tc'8 [\n\td'8\n}"
    assert right.lilypond_format == "\\new Voice {\n\te'8\n\tf'8 ]\n}"
    assert t.lilypond_format == '\\new Voice {\n}'


def test_containertools_split_container_at_index_16():
    '''A single container 'split' at index 0 gives
    an empty lefthand part and a complete righthand part.
    Original container empties contents.
    '''

    t = Staff([Voice("c'8 d'8 e'8 f'8")])
    v = t[0]
    beamtools.BeamSpanner(v)

    r'''
    \new Staff {
        \new Voice {
            c'8 [
            d'8
            e'8
            f'8 ]
        }
    }
    '''

    left, right = containertools.split_container_at_index(v, 0, fracture_spanners=True)

    r'''
    \new Staff {
        \new Voice {
            c'8 [
            d'8
            e'8
            f'8 ]
        }
    }
    '''

    assert left.lilypond_format == '\\new Voice {\n}'
    assert right.lilypond_format == "\\new Voice {\n\tc'8 [\n\td'8\n\te'8\n\tf'8 ]\n}"
    assert v.lilypond_format == '\\new Voice {\n}'
    assert t.lilypond_format == "\\new Staff {\n\t\\new Voice {\n\t\tc'8 [\n\t\td'8\n\t\te'8\n\t\tf'8 ]\n\t}\n}"


def test_containertools_split_container_at_index_17():
    '''Split container at index greater than len(container).
    Lefthand part instantiates with all contents.
    Righthand part instantiates empty.
    Original container empties contents.
    '''

    t = Staff([Voice("c'8 d'8 e'8 f'8")])
    v = t[0]
    beamtools.BeamSpanner(v)

    left, right = containertools.split_container_at_index(v, 10, fracture_spanners=True)

    r'''
    \new Staff {
        \new Voice {
            c'8 [
            d'8
            e'8
            f'8 ]
        }
    }
    '''

    assert left.lilypond_format == "\\new Voice {\n\tc'8 [\n\td'8\n\te'8\n\tf'8 ]\n}"
    assert right.lilypond_format == '\\new Voice {\n}'
    assert v.lilypond_format == '\\new Voice {\n}'
    assert t.lilypond_format == "\\new Staff {\n\t\\new Voice {\n\t\tc'8 [\n\t\td'8\n\t\te'8\n\t\tf'8 ]\n\t}\n}"


def test_containertools_split_container_at_index_18():
    '''Split measure in score and fracture spanners.
    '''

    t = Staff(Measure((2, 8), notetools.make_repeated_notes(2)) * 2)
    pitchtools.set_ascending_named_diatonic_pitches_on_tie_chains_in_expr(t)
    beamtools.BeamSpanner(t[0])
    beamtools.BeamSpanner(t[1])
    slur = spannertools.SlurSpanner(t.leaves)
    measuretools.set_always_format_time_signature_of_measures_in_expr(t)

    r'''
    \new Staff {
        {
            \time 2/8
            c'8 [ (
            d'8 ]
        }
        {
            \time 2/8
            e'8 [
            f'8 ] )
        }
    }
    '''

    left, right = containertools.split_container_at_index(t[0], 1, fracture_spanners=True)
    measuretools.set_always_format_time_signature_of_measures_in_expr(t)

    r'''
    \new Staff {
        {
            \time 1/8
            c'8 [ ] (
        }
        {
            \time 1/8
            d'8 [ ]
        }
        {
            \time 2/8
            e'8 [
            f'8 ] )
        }
    }
    '''

    assert wellformednesstools.is_well_formed_component(t)
    assert t.lilypond_format == "\\new Staff {\n\t{\n\t\t\\time 1/8\n\t\tc'8 [ ] (\n\t}\n\t{\n\t\t\\time 1/8\n\t\td'8 [ ]\n\t}\n\t{\n\t\t\\time 2/8\n\t\te'8 [\n\t\tf'8 ] )\n\t}\n}"


def test_containertools_split_container_at_index_19():
    '''Split left of leaf in score and fracture spanners.
    '''

    t = Staff(Measure((2, 8), notetools.make_repeated_notes(2)) * 2)
    pitchtools.set_ascending_named_diatonic_pitches_on_tie_chains_in_expr(t)
    beamtools.BeamSpanner(t[0])
    beamtools.BeamSpanner(t[1])
    slur = spannertools.SlurSpanner(t.leaves)
    measuretools.set_always_format_time_signature_of_measures_in_expr(t)


    r'''
    \new Staff {
        {
            \time 2/8
            c'8 [ (
            d'8 ]
        }
        {
            \time 2/8
            e'8 [
            f'8 ] )
        }
    }
    '''

    leaf = t.leaves[1]
    left, right = containertools.split_container_at_index(leaf, -100, fracture_spanners=True)

    r'''
    \new Staff {
        {
            \time 2/8
            c'8 ( ) [
            d'8 ] (
        }
        {
            \time 2/8
            e'8 [
            f'8 ] )
        }
    }
    '''

    assert wellformednesstools.is_well_formed_component(t)
    assert left is None
    assert right is leaf
    assert t.lilypond_format == "\\new Staff {\n\t{\n\t\t\\time 2/8\n\t\tc'8 ( ) [\n\t\td'8 ] (\n\t}\n\t{\n\t\t\\time 2/8\n\t\te'8 [\n\t\tf'8 ] )\n\t}\n}"


def test_containertools_split_container_at_index_20():
    '''Split right of leaf in score and fracture spanners.
    '''

    t = Staff(Measure((2, 8), notetools.make_repeated_notes(2)) * 2)
    pitchtools.set_ascending_named_diatonic_pitches_on_tie_chains_in_expr(t)
    beamtools.BeamSpanner(t[0])
    beamtools.BeamSpanner(t[1])
    slur = spannertools.SlurSpanner(t.leaves)
    measuretools.set_always_format_time_signature_of_measures_in_expr(t)

    r'''
    \new Staff {
        {
            \time 2/8
            c'8 [ (
            d'8 ]
        }
        {
            \time 2/8
            e'8 [
            f'8 ] )
        }
    }
    '''

    leaf = t.leaves[1]
    left, right = containertools.split_container_at_index(leaf, 100, fracture_spanners=True)

    r'''
    \new Staff {
        {
            \time 2/8
            c'8 [ (
            d'8 ] )
        }
        {
            \time 2/8
            e'8 [ (
            f'8 ] )
        }
    }
    '''

    assert wellformednesstools.is_well_formed_component(t)
    assert left is leaf
    assert right is None
    assert t.lilypond_format == "\\new Staff {\n\t{\n\t\t\\time 2/8\n\t\tc'8 [ (\n\t\td'8 ] )\n\t}\n\t{\n\t\t\\time 2/8\n\t\te'8 [ (\n\t\tf'8 ] )\n\t}\n}"


def test_containertools_split_container_at_index_21():
    '''Split in-score measure without power-of-two time signature denominator.
    Fractured spanners but do not tie over split locus.
    Measure contents necessitate denominator change.
    '''

    t = Staff([Measure((3, 12), "c'8. d'8.")])
    beamtools.BeamSpanner(t[0])
    spannertools.SlurSpanner(t.leaves)
    measuretools.set_always_format_time_signature_of_measures_in_expr(t)

    r'''
    \new Staff {
        {
            \time 3/12
            \scaleDurations '(2 . 3) {
                c'8. [ (
                d'8. ] )
            }
        }
    }
    '''

    halves = containertools.split_container_at_index(t[0], 1, fracture_spanners=True)
    measuretools.set_always_format_time_signature_of_measures_in_expr(t)

    r'''
    \new Staff {
        {
            \time 3/24
            \scaleDurations '(2 . 3) {
                c'8. [ ] (
            }
        }
        {
            \time 3/24
            \scaleDurations '(2 . 3) {
                d'8. [ ] )
            }
        }
    }
    '''

    assert wellformednesstools.is_well_formed_component(t)
    assert len(halves) == 2
    assert t.lilypond_format == "\\new Staff {\n\t{\n\t\t\\time 3/24\n\t\t\\scaleDurations #'(2 . 3) {\n\t\t\tc'8. [ ] (\n\t\t}\n\t}\n\t{\n\t\t\\time 3/24\n\t\t\\scaleDurations #'(2 . 3) {\n\t\t\td'8. [ ] )\n\t\t}\n\t}\n}"



def test_containertools_split_container_at_index_22():
    '''Split in-score measure with power-of-two time signature denominator.
    Fractured spanners but do not tie over split locus.
    Measure contents necessitate denominator change.
    '''

    t = Staff([Measure((3, 8), "c'8. d'8.")])
    beamtools.BeamSpanner(t[0])
    spannertools.SlurSpanner(t.leaves)
    measuretools.set_always_format_time_signature_of_measures_in_expr(t)

    r'''
    \new Staff {
        {
            \time 3/8
            c'8. [ (
            d'8. ] )
        }
    }
    '''

    halves = containertools.split_container_at_index(t[0], 1, fracture_spanners=True)
    measuretools.set_always_format_time_signature_of_measures_in_expr(t)

    r'''
    \new Staff {
        {
            \time 3/16
            c'8. [ ] (
        }
        {
            \time 3/16
            d'8. [ ] )
        }
    }
    '''

    assert wellformednesstools.is_well_formed_component(t)
    assert len(halves) == 2
    assert t.lilypond_format == "\\new Staff {\n\t{\n\t\t\\time 3/16\n\t\tc'8. [ ] (\n\t}\n\t{\n\t\t\\time 3/16\n\t\td'8. [ ] )\n\t}\n}"



import abc
import copy
import types
from abjad.tools import beamtools
from abjad.tools import durationtools
from abjad.tools import iterationtools
from abjad.tools import leaftools
from abjad.tools import mathtools
from abjad.tools import sequencetools
from abjad.tools import tietools
from abjad.tools.rhythmmakertools.RhythmMaker import RhythmMaker


class BurnishedRhythmMaker(RhythmMaker):
    '''.. versionadded:: 2.8

    Abstract base class for rhythm-makers that burnish some or
    all of the output cells they produce.

    'Burnishing' means to forcibly cast the first or last 
    (or both first and last) elements of a output cell to be 
    either a note or rest.

    'Division-burnishing' rhythm-makers burnish every output cell they produce.

    'Output-burnishing' rhythm-makers burnish only the first and last 
    output cells they produce and leave interior output cells unchanged.
    '''

    ### CLASS ATTRIBUTES ###

    __metaclass__ = abc.ABCMeta

    _default_positional_input_arguments = ([-1, 4, -2, 3], 16, )

    ### INITIALIZER ###

    def __init__(self, talea, talea_denominator, prolation_addenda=None,
        lefts=None, middles=None, rights=None, left_lengths=None, right_lengths=None,
        secondary_divisions=None,
        talea_helper=None, prolation_addenda_helper=None,
        lefts_helper=None, middles_helper=None, rights_helper=None,
        left_lengths_helper=None, right_lengths_helper=None, secondary_divisions_helper=None,
        beam_each_cell=False, beam_cells_together=False,
        decrease_durations_monotonically=True, tie_split_notes=False, tie_rests=False
        ):
        RhythmMaker.__init__(self, 
            beam_each_cell=beam_each_cell, 
            beam_cells_together=beam_cells_together)
        prolation_addenda = self._none_to_new_list(prolation_addenda)
        lefts = self._none_to_new_list(lefts)
        middles = self._none_to_new_list(middles)
        rights = self._none_to_new_list(rights)
        left_lengths = self._none_to_new_list(left_lengths)
        right_lengths = self._none_to_new_list(right_lengths)
        secondary_divisions = self._none_to_new_list(secondary_divisions)
        talea_helper = self._none_to_trivial_helper(talea_helper)
        prolation_addenda_helper = self._none_to_trivial_helper(prolation_addenda_helper)
        lefts_helper = self._none_to_trivial_helper(lefts_helper)
        middles_helper = self._none_to_trivial_helper(middles_helper)
        rights_helper = self._none_to_trivial_helper(rights_helper)
        left_lengths_helper = self._none_to_trivial_helper(left_lengths_helper)
        right_lengths_helper = self._none_to_trivial_helper(right_lengths_helper)
        secondary_divisions_helper = self._none_to_trivial_helper(secondary_divisions_helper)
        assert sequencetools.all_are_integer_equivalent_numbers(talea)
        assert mathtools.is_positive_integer_equivalent_number(talea_denominator)
        assert sequencetools.all_are_nonnegative_integer_equivalent_numbers(prolation_addenda)
        assert all([x in (-1, 0, 1) for x in lefts])
        assert all([x in (-1, 0, 1) for x in middles])
        assert all([x in (-1, 0, 1) for x in rights])
        assert sequencetools.all_are_nonnegative_integer_equivalent_numbers(left_lengths)
        assert sequencetools.all_are_nonnegative_integer_equivalent_numbers(right_lengths)
        assert sequencetools.all_are_nonnegative_integer_equivalent_numbers(secondary_divisions)
        assert isinstance(talea_helper, (types.FunctionType, types.MethodType))
        assert isinstance(prolation_addenda_helper, (types.FunctionType, types.MethodType))
        assert isinstance(lefts_helper, (types.FunctionType, types.MethodType))
        assert isinstance(middles_helper, (types.FunctionType, types.MethodType))
        assert isinstance(rights_helper, (types.FunctionType, types.MethodType))
        assert isinstance(left_lengths_helper, (types.FunctionType, types.MethodType))
        assert isinstance(right_lengths_helper, (types.FunctionType, types.MethodType))
        assert isinstance(decrease_durations_monotonically, bool)
        assert isinstance(tie_split_notes, bool)
        assert isinstance(tie_rests, bool)
        self.talea = talea
        self.talea_denominator = talea_denominator
        self.prolation_addenda = prolation_addenda
        self.lefts = lefts
        self.middles = middles
        self.rights = rights
        self.left_lengths = left_lengths
        self.right_lengths = right_lengths
        self.secondary_divisions = secondary_divisions
        self.talea_helper = talea_helper
        self.prolation_addenda_helper = prolation_addenda_helper
        self.lefts_helper = lefts_helper
        self.middles_helper = middles_helper
        self.rights_helper = rights_helper
        self.left_lengths_helper = left_lengths_helper
        self.right_lengths_helper = right_lengths_helper
        self.secondary_divisions_helper = secondary_divisions_helper
        #self.beam_each_cell = beam_each_cell
        self.decrease_durations_monotonically = decrease_durations_monotonically
        self.tie_split_notes = tie_split_notes
        self.tie_rests = tie_rests

    ### SPECIAL METHODS ###

    def __call__(self, divisions, seeds=None):
        duration_pairs, seeds = RhythmMaker.__call__(self, divisions, seeds)
        octuplet = self._prepare_input(seeds)
        talea, prolation_addenda = octuplet[:2]
        secondary_divisions = octuplet[-1]
        talee = (talea, prolation_addenda, secondary_divisions)
        result = self._scale_talee(duration_pairs, self.talea_denominator, talee)
        duration_pairs, lcd, talea, prolation_addenda, secondary_divisions = result
        secondary_duration_pairs = self._make_secondary_duration_pairs(
            duration_pairs, secondary_divisions)
        septuplet = (talea, prolation_addenda) + octuplet[2:-1]
        numeric_map = self._make_numeric_map(secondary_duration_pairs, septuplet)
        leaf_lists = self._make_leaf_lists(numeric_map, lcd)
        if not prolation_addenda:
            result = leaf_lists
        else:
            tuplets = self._make_tuplets(secondary_duration_pairs, leaf_lists)
            result = tuplets
        if self.beam_each_cell:
            for cell in result:
                beamtools.MultipartBeamSpanner(cell)
        if self.tie_split_notes:
            self._add_ties(result)
        return result

    def __eq__(self, other):
        return isinstance(other, type(self)) and all([
            self.talea == other.talea,
            self.talea_denominator == other.talea_denominator,
            self.prolation_addenda == other.prolation_addenda,
            self.lefts == other.lefts,
            self.middles == other.middles,
            self.rights == other.rights,
            self.left_lengths == other.left_lengths,
            self.right_lengths == other.right_lengths,
            self.secondary_divisions == other.secondary_divisions,
            #self.talea_helper == other.talea_helper,
            #self.prolation_addenda_helper == other.prolation_addenda_helper,
            #self.lefts_helper == other.lefts_helper,
            #self.middles_helper == other.middles_helper,
            #self.rights_helper == other.rights_helper,
            #self.left_lengths_helper == other.left_lengths_helper,
            #self.right_lengths_helper == other.right_lengths_helper,
            #self.secondary_divisons_helper == other.secondary_divisions_helper,
            ])    

    ### PRIVATE METHODS ###

    def _add_ties(self, result):
        leaves = list(iterationtools.iterate_leaves_in_expr(result))
        written_durations = leaftools.list_written_durations_of_leaves_in_expr(leaves)
        weights = [durationtools.Duration(numerator, self.talea_denominator) for numerator in self.talea]
        parts = sequencetools.partition_sequence_by_weights_exactly(
            written_durations, weights=weights, cyclic=True, overhang=True)
        counts = [len(part) for part in parts]
        parts = sequencetools.partition_sequence_by_counts(leaves, counts)
        for part in parts:
            tie_spanner = tietools.TieSpanner()
            # this is voodoo to temporarily neuter the contiguity constraint
            tie_spanner._contiguity_constraint = None
            tietools.remove_tie_spanners_from_components_in_expr(part)
            tie_spanner.extend(part)

    def _burnish_division_part(self, division_part, indicator):
        assert len(division_part) == len(indicator)
        new_division_part = []
        for number, i in zip(division_part, indicator):
            if i == -1:
                new_division_part.append(-abs(number))
            elif i == 0:
                new_division_part.append(number)
            elif i == 1:
                new_division_part.append(abs(number))
            else:
                raise ValueError
        new_division_part = type(division_part)(new_division_part)
        return new_division_part

    @abc.abstractmethod
    def _burnish_division_parts(self, divisions, quintuplet):
        pass

    def _make_leaf_lists(self, numeric_map, talea_denominator):
        leaf_lists = []
        for map_division in numeric_map:
            leaf_list = leaftools.make_leaves_from_talea(
                map_division, talea_denominator, 
                decrease_durations_monotonically=self.decrease_durations_monotonically, 
                tie_rests=self.tie_rests)
            leaf_lists.append(leaf_list)
        return leaf_lists

    def _make_numeric_map(self, duration_pairs, septuplet):
        talea, prolation_addenda, lefts, middles, rights, left_lengths, right_lengths = septuplet
        prolated_duration_pairs = self._make_prolated_duration_pairs(
            duration_pairs, prolation_addenda)
        if isinstance(prolated_duration_pairs[0], tuple):
            prolated_numerators = [pair[0] for pair in prolated_duration_pairs]
        else:
            prolated_numerators = [pair.numerator for pair in prolated_duration_pairs]
        map_divisions = sequencetools.split_sequence_extended_to_weights(
            talea, prolated_numerators, overhang=False)
        quintuplet = (lefts, middles, rights, left_lengths, right_lengths)
        burnished_map_divisions = self._burnish_division_parts(map_divisions, quintuplet)
        numeric_map = burnished_map_divisions
        return numeric_map

    def _make_prolated_duration_pairs(self, duration_pairs, prolation_addenda):
        prolated_duration_pairs = []
        for i, duration_pair in enumerate(duration_pairs):
            if not prolation_addenda:
                prolated_duration_pairs.append(duration_pair)
            else:
                prolation_addendum = prolation_addenda[i]
                if hasattr(duration_pair, 'numerator'):
                    prolation_addendum %= duration_pair.numerator
                else:
                    prolation_addendum %= duration_pair[0]
                if isinstance(duration_pair, tuple):
                    numerator, denominator = duration_pair
                else:
                    numerator, denominator = duration_pair.pair
                prolated_duration_pair = (numerator + prolation_addendum, denominator)
                prolated_duration_pairs.append(prolated_duration_pair)
        return prolated_duration_pairs

    def _prepare_input(self, seeds):
        talea = sequencetools.CyclicTuple(self.talea_helper(self.talea, seeds))
        prolation_addenda = self.prolation_addenda_helper(self.prolation_addenda, seeds)
        prolation_addenda = sequencetools.CyclicTuple(prolation_addenda)
        lefts = sequencetools.CyclicTuple(self.lefts_helper(self.lefts, seeds))
        middles = sequencetools.CyclicTuple(self.middles_helper(self.middles, seeds))
        rights = sequencetools.CyclicTuple(self.rights_helper(self.rights, seeds))
        left_lengths = sequencetools.CyclicTuple(self.left_lengths_helper(self.left_lengths, seeds))
        right_lengths = sequencetools.CyclicTuple(self.right_lengths_helper(self.right_lengths, seeds))
        secondary_divisions = self.secondary_divisions_helper(self.secondary_divisions, seeds)
        secondary_divisions = sequencetools.CyclicTuple(secondary_divisions)
        return talea, prolation_addenda, \
            lefts, middles, rights, left_lengths, right_lengths, secondary_divisions

    ### PUBLIC METHODS ###

    
    def reverse(self):
        '''.. versionadded:: 2.10

        Reverse burnished rhythm-maker.

        .. note:: method is provisional.

        Defined equal to reversal of the following on a copy of rhythm-maker::

            new.talea
            new.prolation_addenda
            new.lefts
            new.middles
            new.rights
            new.left_lengths
            new.right_lengths
            new.secondary_divisions

        Return newly constructed rhythm-maker.
        '''
        new = copy.deepcopy(self)
        new.talea.reverse()
        new.prolation_addenda.reverse()
        new.lefts.reverse()
        new.middles.reverse()
        new.rights.reverse()
        new.left_lengths.reverse()
        new.right_lengths.reverse()
        new.secondary_divisions.reverse()
        if new.decrease_durations_monotonically:
            new.decrease_durations_monotonically = False
        return new



from abjad import *
from abjad.tools import quantizationtools


def test_quantizationtools_tempo_scaled_leaves_to_q_events_01():

    staff = Staff([])

    staff.append(Note(0, (1, 4)))
    staff.append(Rest((1, 4)))
    staff.append(Rest((1, 8)))
    staff.append(Note(1, (1, 8)))
    staff.append(Note(1, (1, 8)))
    staff.append(Note(2, (1, 8)))
    staff.append(Note(2, (1, 8)))
    staff.append(Note(3, (1, 8)))
    staff.append(skiptools.Skip((1, 4)))
    staff.append(Rest((1, 4)))
    staff.append(Note(3, (1, 8)))
    staff.append(Chord([0, 1, 4], (1, 4)))

    tietools.TieSpanner(staff[3:5])
    tietools.TieSpanner(staff[5:7])
    tietools.TieSpanner(staff[7:11])

    tempo = contexttools.TempoMark((1, 4), 55)

    q_events = quantizationtools.tempo_scaled_leaves_to_q_events(staff.leaves, tempo)

    assert q_events == [
        quantizationtools.PitchedQEvent(
            durationtools.Offset(0, 1),
            (pitchtools.NamedChromaticPitch("c'"),)
            ),
        quantizationtools.SilentQEvent(
            durationtools.Offset(12000, 11)
            ),
        quantizationtools.PitchedQEvent(
            durationtools.Offset(30000, 11),
            (pitchtools.NamedChromaticPitch("cs'"),)
            ),
        quantizationtools.PitchedQEvent(
            durationtools.Offset(42000, 11),
            (pitchtools.NamedChromaticPitch("d'"),)
            ),
        quantizationtools.PitchedQEvent(
            durationtools.Offset(54000, 11),
            (pitchtools.NamedChromaticPitch("ef'"),)
            ),
        quantizationtools.SilentQEvent(
            durationtools.Offset(60000, 11)
            ),
        quantizationtools.PitchedQEvent(
            durationtools.Offset(84000, 11),
            (pitchtools.NamedChromaticPitch("ef'"),)
            ),
        quantizationtools.PitchedQEvent(
            durationtools.Offset(90000, 11),
            (
                pitchtools.NamedChromaticPitch("c'"),
                pitchtools.NamedChromaticPitch("cs'"),
                pitchtools.NamedChromaticPitch("e'"),
            )
            ),
        quantizationtools.TerminalQEvent(
            durationtools.Offset(102000, 11)
            )
    ]


def test_quantizationtools_tempo_scaled_leaves_to_q_events_02():

    staff = Staff([])

    staff.append(Note(0, (1, 4)))
    staff.append(Rest((1, 4)))
    staff.append(Rest((1, 8)))
    staff.append(Note(1, (1, 8)))
    staff.append(Note(1, (1, 8)))
    staff.append(Note(2, (1, 8)))
    staff.append(Note(2, (1, 8)))
    staff.append(Note(3, (1, 8)))
    staff.append(skiptools.Skip((1, 4)))
    staff.append(Rest((1, 4)))
    staff.append(Note(3, (1, 8)))
    staff.append(Chord([0, 1, 4], (1, 4)))

    tietools.TieSpanner(staff[3:5])
    tietools.TieSpanner(staff[5:7])
    tietools.TieSpanner(staff[7:11])

    contexttools.TempoMark((1, 4), 58, target_context=Staff)(staff[0])
    contexttools.TempoMark((1, 4), 77, target_context=Staff)(staff[9])

    q_events = quantizationtools.tempo_scaled_leaves_to_q_events(staff.leaves)

    assert q_events == [
        quantizationtools.PitchedQEvent(
            durationtools.Offset(0, 1),
            (pitchtools.NamedChromaticPitch("c'"),)
            ),
        quantizationtools.SilentQEvent(
            durationtools.Offset(30000, 29)
            ),
        quantizationtools.PitchedQEvent(
            durationtools.Offset(75000, 29),
            (pitchtools.NamedChromaticPitch("cs'"),)
            ),
        quantizationtools.PitchedQEvent(
            durationtools.Offset(105000, 29),
            (pitchtools.NamedChromaticPitch("d'"),)
            ),
        quantizationtools.PitchedQEvent(
            durationtools.Offset(135000, 29),
            (pitchtools.NamedChromaticPitch("ef'"),)
            ),
        quantizationtools.SilentQEvent(
            durationtools.Offset(150000, 29)
            ),
        quantizationtools.PitchedQEvent(
            durationtools.Offset(15600000, 2233),
            (pitchtools.NamedChromaticPitch("ef'"),)
            ),
        quantizationtools.PitchedQEvent(
            durationtools.Offset(16470000, 2233),
            (
                pitchtools.NamedChromaticPitch("c'"),
                pitchtools.NamedChromaticPitch("cs'"),
                pitchtools.NamedChromaticPitch("e'"),
            )
            ),
        quantizationtools.TerminalQEvent(
            durationtools.Offset(18210000, 2233)
            )
    ]



def chromatic_pitch_number_to_diatonic_pitch_class_number(chromatic_pitch_number):
    '''.. versionadded:: 2.0

    Change `chromatic_pitch_number` to diatonic pitch-class number::

        >>> pitchtools.chromatic_pitch_number_to_diatonic_pitch_class_number(13)
        0

    Return integer.
    '''
    from abjad.tools import pitchtools

    chromatic_pitch_class_number = chromatic_pitch_number % 12

    return pitchtools.chromatic_pitch_class_number_to_diatonic_pitch_class_number(
        chromatic_pitch_class_number)



import math
import re
from fractions import Fraction
from abjad.tools.abctools import ImmutableAbjadObject
from abjad.tools import mathtools


class Duration(ImmutableAbjadObject, Fraction):
    r'''.. versionadded:: 2.0

    Abjad model of musical duration.

    Initialize from integer numerator::

        >>> Duration(3)
        Duration(3, 1)

    Initialize from integer numerator and denominator::

        >>> Duration(3, 16)
        Duration(3, 16)

    Initialize from integer-equivalent numeric numerator::

        >>> Duration(3.0)
        Duration(3, 1)

    Initialize from integer-equivalent numeric numerator and denominator::

        >>> Duration(3.0, 16)
        Duration(3, 16)

    Initialize from integer-equivalent singleton::

        >>> Duration((3,))
        Duration(3, 1)

    Initialize from integer-equivalent pair::

        >>> Duration((3, 16))
        Duration(3, 16)

    Initialize from other duration::

        >>> Duration(Duration(3, 16))
        Duration(3, 16)

    Intialize from fraction::

        >>> Duration(Fraction(3, 16))
        Duration(3, 16)

    Initialize from solidus string::
        
        >>> Duration('3/16')
        Duration(3, 16)

    initialize from LilyPond duration string::

        >>> Duration('8.')
        Duration(3, 16)

    Initialize from nonreduced fraction::

        >>> Duration(mathtools.NonreducedFraction(3, 16))
        Duration(3, 16)
        
    Durations inherit from built-in fraction::

        >>> isinstance(Duration(3, 16), Fraction)
        True

    Durations are numeric::

        >>> import numbers

    ::

        >>> isinstance(Duration(3, 16), numbers.Number)
        True

    Durations are immutable.
    '''

    def __new__(klass, *args):
        from abjad.tools import sequencetools
        if len(args) == 1 and sequencetools.is_integer_equivalent_singleton(args[0]):
            self = Fraction.__new__(klass, int(args[0][0]))
        elif len(args) == 1 and sequencetools.is_fraction_equivalent_pair(args[0]):
            self = Fraction.__new__(klass, int(args[0][0]), int(args[0][1]))
        elif len(args) == 1 and isinstance(args[0], str) and not '/' in args[0]:
            result = Duration._init_from_lilypond_duration_string(args[0])
            self = Fraction.__new__(klass, result)
        elif len(args) == 1 and hasattr(args[0], 'numerator') and hasattr(args[0], 'denominator'):
            self = Fraction.__new__(klass, args[0].numerator, args[0].denominator)
        elif sequencetools.all_are_integer_equivalent_numbers(args):
            self = Fraction.__new__(klass, *[int(x) for x in args])
        else:
            self = Fraction.__new__(klass, *args)
        return self

    ### SPECIAL METHODS ###

    def __abs__(self, *args):
        return type(self)(Fraction.__abs__(self, *args))

    def __add__(self, *args):
        if len(args) == 1 and isinstance(args[0], mathtools.NonreducedFraction):
            return args[0].__radd__(self)
        else:
            return type(self)(Fraction.__add__(self, *args))

    def __div__(self, *args):
        from abjad.tools import durationtools
        if len(args) == 1 and isinstance(args[0], type(self)):
            return durationtools.Multiplier(Fraction.__div__(self, *args))
        elif len(args) == 1 and isinstance(args[0], mathtools.NonreducedFraction):
            return args[0].__rdiv__(self)
        else:
            return type(self)(Fraction.__div__(self, *args))

    def __divmod__(self, *args):
        return type(self)(Fraction.__divmod__(self, *args))

    def __eq__(self, arg):
        return Fraction.__eq__(self, arg)

    def __ge__(self, arg):
        return Fraction.__ge__(self, arg)

    def __gt__(self, arg):
        return Fraction.__gt__(self, arg)        

    def __le__(self, arg):
        return Fraction.__le__(self, arg)

    def __lt__(self, arg):
        return Fraction.__lt__(self, arg)

    def __mod__(self, *args):
        return type(self)(Fraction.__mod__(self, *args))

    def __mul__(self, *args):
        if len(args) == 1 and isinstance(args[0], mathtools.NonreducedFraction):
            return args[0].__rmul__(self)
        else:
            return type(self)(Fraction.__mul__(self, *args))

    def __ne__(self, arg):
        return Fraction.__ne__(self, arg)

    def __neg__(self, *args):
        return type(self)(Fraction.__neg__(self, *args))

    def __pos__(self, *args):
        return type(self)(Fraction.__pos__(self, *args))

    def __pow__(self, *args):
        return type(self)(Fraction.__pow__(self, *args))

    def __radd__(self, *args):
        return type(self)(Fraction.__radd__(self, *args))

    def __rdiv__(self, *args):
        return type(self)(Fraction.__rdiv__(self, *args))

    def __rdivmod__(self, *args):
        return type(self)(Fraction.__rdivmod__(self, *args))

    def __reduce__(self):
        return self.__class__, (self.numerator, self.denominator)

    def __reduce_ex__(self, protocol):
        return self.__class__, (self.numerator, self.denominator)

    def __repr__(self):
        return '%s(%s, %s)' % (type(self).__name__, self.numerator, self.denominator)

    def __rmod__(self, *args):
        return type(self)(Fraction.__rmod__(self, *args))

    def __rmul__(self, *args):
        return type(self)(Fraction.__rmul__(self, *args))

    def __rpow__(self, *args):
        return type(self)(Fraction.__rpow__(self, *args))

    def __rsub__(self, *args):
        return type(self)(Fraction.__rsub__(self, *args))

    def __rtruediv__(self, *args):
        return type(self)(Fraction.__rtruediv__(self, *args))

    def __sub__(self, *args):
        if len(args) == 1 and isinstance(args[0], mathtools.NonreducedFraction):
            return args[0].__rsub__(self)
        else:
            return type(self)(Fraction.__sub__(self, *args))

    def __truediv__(self, *args):
        return type(self)(Fraction.__truediv__(self, *args))

    ### READ-ONLY PRIVATE PROPERTIES ###

    @property
    def _keyword_argument_names(self):
        return ()

    @property
    def _positional_argument_values(self):
        result = []
        result.append(self.numerator)
        result.append(self.denominator)
        return tuple(result)

    ### PRIVATE METHODS ###

    # do not indent in storage
    def _get_tools_package_qualified_repr_pieces(self, is_indented=True):
        return [''.join(
            ImmutableAbjadObject._get_tools_package_qualified_repr_pieces(self, is_indented=False))]

    @staticmethod
    def _init_from_lilypond_duration_string(duration_string):
        numeric_body_strings = [str(2 ** n) for n in range(8)]
        other_body_strings = [r'\\breve', r'\\longa', r'\\maxima']
        body_strings = numeric_body_strings + other_body_strings
        body_strings = '|'.join(body_strings)
        pattern = r'^(%s)(\.*)$' % body_strings

        match = re.match(pattern, duration_string)
        if match is None:
            raise DurationError('incorrect duration string format: %s.' %
                duration_string)
        body_string, dots_string = match.groups()

        try:
            body_denominator = int(body_string)
            body_duration = Fraction(1, body_denominator)
        except ValueError:
            if body_string == r'\breve':
                body_duration = Fraction(2)
            elif body_string == r'\longa':
                body_duration = Fraction(4)
            elif body_string == r'\maxima':
                body_duration = Fraction(8)
            else:
                raise ValueError('unknown body string %s.' % body_string)

        rational = body_duration
        for n in range(len(dots_string)):
            exponent = n + 1
            denominator = 2 ** exponent
            multiplier = Fraction(1, denominator)
            addend = multiplier * body_duration
            rational += addend

        return rational

    ### READ-ONLY PUBLIC PROPERTIES ###

    @property
    def dot_count(self):
        r'''.. versionadded:: 2.11

        Positive integer number of dots required to notate duration::

            >>> for n in range(1, 16 + 1):
            ...     try:
            ...         duration = Duration(n, 16)
            ...         print '{}\t{}'.format(duration.with_denominator(16), duration.dot_count)
            ...     except AssignabilityError:
            ...         print '{}\t{}'.format(duration.with_denominator(16), '--')
            ... 
            1/16    0
            2/16    0
            3/16    1
            4/16    0
            5/16    --
            6/16    1
            7/16    2
            8/16    0
            9/16    --
            10/16   --
            11/16   --
            12/16   1
            13/16   --
            14/16   2
            15/16   3
            16/16   0

        Return positive integer.

        Raise assignability error when duration is not assignable.
        '''
        if not self.is_assignable:
            raise AssignabilityError

        binary_string = mathtools.integer_to_binary_string(self.numerator)
        digit_sum = sum([int(x) for x in list(binary_string)])
        dot_count = digit_sum - 1
        return dot_count

    @property
    def equal_or_greater_assignable(self):
        r'''Equal or greater assignable::

            >>> for numerator in range(1, 16 + 1):
            ...     duration = Duration(numerator, 16)
            ...     result = duration.equal_or_greater_assignable
            ...     print '{}\t{}'.format(duration.with_denominator(16), result)
            ... 
            1/16    1/16
            2/16    1/8
            3/16    3/16
            4/16    1/4
            5/16    3/8
            6/16    3/8
            7/16    7/16
            8/16    1/2
            9/16    3/4
            10/16   3/4
            11/16   3/4
            12/16   3/4
            13/16   7/8
            14/16   7/8
            15/16   15/16
            16/16   1

        Return new duration.
        '''
        good_denominator = mathtools.greatest_power_of_two_less_equal(self.denominator)
        current_numerator = self.numerator
        candidate = type(self)(current_numerator, good_denominator)
        while not candidate.is_assignable:
            current_numerator += 1
            candidate = type(self)(current_numerator, good_denominator)
        return candidate

    @property
    def equal_or_greater_power_of_two(self):
        r'''Equal or greater power of ``2``::

            >>> for numerator in range(1, 16 + 1):
            ...     duration = Duration(numerator, 16)
            ...     result = duration.equal_or_greater_power_of_two
            ...     print '{}\t{}'.format(duration.with_denominator(16), result)
            ... 
            1/16    1/16
            2/16    1/8
            3/16    1/4
            4/16    1/4
            5/16    1/2
            6/16    1/2
            7/16    1/2
            8/16    1/2
            9/16    1
            10/16   1
            11/16   1
            12/16   1
            13/16   1
            14/16   1
            15/16   1
            16/16   1

        Return new duration.
        '''
        denominator_exponent = -int(math.ceil(math.log(self, 2)))
        return type(self)(1, 2) ** denominator_exponent

    @property
    def equal_or_lesser_assignable(self):
        r'''Equal or lesser assignable::

            >>> for numerator in range(1, 16 + 1):
            ...     duration = Duration(numerator, 16)
            ...     result = duration.equal_or_lesser_assignable
            ...     print '{}\t{}'.format(duration.with_denominator(16), result)
            ... 
            1/16    1/16
            2/16    1/8
            3/16    3/16
            4/16    1/4
            5/16    1/4
            6/16    3/8
            7/16    7/16
            8/16    1/2
            9/16    1/2
            10/16   1/2
            11/16   1/2
            12/16   3/4
            13/16   3/4
            14/16   7/8
            15/16   15/16
            16/16   1

        Return new duration.
        '''
        good_denominator = mathtools.least_power_of_two_greater_equal(self.denominator)
        current_numerator = self.numerator
        candidate = type(self)(current_numerator, good_denominator)
        while not candidate.is_assignable:
            current_numerator -= 1
            candidate = type(self)(current_numerator, good_denominator)
        return candidate

    @property
    def equal_or_lesser_power_of_two(self):
        r'''Equal or lesser power of ``2``::

            >>> for numerator in range(1, 16 + 1):
            ...     duration = Duration(numerator, 16)
            ...     result = duration.equal_or_lesser_power_of_two
            ...     print '{}\t{}'.format(duration.with_denominator(16), result)
            ... 
            1/16    1/16
            2/16    1/8
            3/16    1/8
            4/16    1/4
            5/16    1/4
            6/16    1/4
            7/16    1/4
            8/16    1/2
            9/16    1/2
            10/16   1/2
            11/16   1/2
            12/16   1/2
            13/16   1/2
            14/16   1/2
            15/16   1/2
            16/16   1

        Return new duration.
        '''
        denominator_exponent = -int(math.floor(math.log(self, 2)))
        return type(self)(1, 2) ** denominator_exponent
        
    @property
    def flag_count(self):
        r'''.. versionadded:: 2.11

        Nonnegative integer number of flags required to notate duration::

            >>> for n in range(1, 16 + 1):
            ...     duration = Duration(n, 64)
            ...     print '{}\t{}'.format(duration.with_denominator(64), duration.flag_count)
            ... 
            1/64    4
            2/64    3
            3/64    3
            4/64    2
            5/64    2
            6/64    2
            7/64    2
            8/64    1
            9/64    1
            10/64   1
            11/64   1
            12/64   1
            13/64   1
            14/64   1
            15/64   1
            16/64   0

        Return nonnegative integer.
        '''
        # TODO: rewrite with only one operation per line
        flag_count = max(-int(math.floor(math.log(float(self.numerator) /
            self.denominator, 2))) - 2, 0)
        return flag_count

    @property
    def has_power_of_two_denominator(self):
        r'''.. versionadded:: 2.11

        True when duration is an integer power of ``2``.
        Otherwise false::

            >>> for n in range(1, 16 + 1):
            ...     duration = Duration(1, n)
            ...     print '{}\t{}'.format(duration, duration.has_power_of_two_denominator)
            ...
            1       True
            1/2     True
            1/3     False
            1/4     True
            1/5     False
            1/6     False
            1/7     False
            1/8     True
            1/9     False
            1/10    False
            1/11    False
            1/12    False
            1/13    False
            1/14    False
            1/15    False
            1/16    True

        Return boolean.
        '''
        exponent = math.log(self.denominator, 2)
        return int(exponent) == exponent

    @property
    def implied_prolation(self):
        r'''.. versionadded:: 2.11

        Implied prolation of multiplier::

            >>> for denominator in range(1, 16 + 1):
            ...     multiplier = Multiplier(1, denominator)
            ...     print '{}\t{}'.format(multiplier, multiplier.implied_prolation)
            ... 
            1       1
            1/2     1
            1/3     2/3
            1/4     1
            1/5     4/5
            1/6     2/3
            1/7     4/7
            1/8     1
            1/9     8/9
            1/10    4/5
            1/11    8/11
            1/12    2/3
            1/13    8/13
            1/14    4/7
            1/15    8/15
            1/16    1

        Return new multipler.
        '''
        from abjad.tools import durationtools
        numerator = mathtools.greatest_power_of_two_less_equal(self.denominator)
        return durationtools.Multiplier(numerator, self.denominator)

    @property
    def is_assignable(self):
        r'''.. versionadded:: 2.11

        True when assignable. Otherwise false::

            >>> for numerator in range(0, 16 + 1):
            ...     duration = Duration(numerator, 16)
            ...     print '{}\t{}'.format(duration.with_denominator(16), duration.is_assignable)
            ... 
            0/16    False
            1/16    True
            2/16    True
            3/16    True
            4/16    True
            5/16    False
            6/16    True
            7/16    True
            8/16    True
            9/16    False
            10/16   False
            11/16   False
            12/16   True
            13/16   False
            14/16   True
            15/16   True
            16/16   True

        Return boolean.
        '''
        if 0 < self < 16:
            if mathtools.is_nonnegative_integer_power_of_two(self.denominator):
                if mathtools.is_assignable_integer(self.numerator):
                    return True
        return False

    @property
    def lilypond_duration_string(self):
        r'''.. versionadded:: 2.11

        LilyPond duration string of assignable duration.

            >>> Duration(3, 16).lilypond_duration_string
            '8.'

        Return string.

        Raise assignability error when duration is not assignable.
        '''
        from abjad.tools import durationtools

        if not self.is_assignable:
            raise AssignabilityError

        undotted_rational = self.equal_or_lesser_power_of_two
        if undotted_rational <= 1:
            undotted_duration_string = str(undotted_rational.denominator)
        elif undotted_rational == type(self)(2, 1):
            undotted_duration_string = r'\breve'
        elif undotted_rational == type(self)(4, 1):
            undotted_duration_string = r'\longa'
        elif undotted_rational == type(self)(8, 1):
            undotted_duration_string = r'\maxima'
        else:
            raise ValueError('can not process undotted rational: %s' % undotted_rational)

        dot_count = self.dot_count
        dot_string = '.' * dot_count
        dotted_duration_string = undotted_duration_string + dot_string
        return dotted_duration_string

    @property
    def pair(self):
        '''.. versionadded:: 2.9

        Read-only pair of duration numerator and denominator::

            >>> duration = Duration(3, 16)

        ::

            >>> duration.pair
            (3, 16)

        Return integer pair.
        '''
        return self.numerator, self.denominator

    @property
    def prolation_string(self):
        r'''.. versionadded:: 2.11

        Prolation string.

            >>> generator = durationtools.yield_durations(unique=True)
            >>> for n in range(16):
            ...     rational = generator.next()
            ...     duration = Duration(rational)
            ...     print '{}\t{}'.format(duration, duration.prolation_string)
            ...
            1       1:1
            2       1:2
            1/2     2:1
            1/3     3:1
            3       1:3
            4       1:4
            3/2     2:3
            2/3     3:2
            1/4     4:1
            1/5     5:1
            5       1:5
            6       1:6
            5/2     2:5
            4/3     3:4
            3/4     4:3
            2/5     5:2

        Return string.
        '''
        return '{}:{}'.format(self.denominator, self.numerator)
    
    @property
    def reciprocal(self):
        '''.. versionadded:: 2.11

        Reciprocal of duration.

        Return newly constructed duration.
        '''
        return type(self)(self.denominator, self.numerator)

    ### PUBLIC FUNCTIONS ###

    @staticmethod
    def is_token(expr):
        '''.. versionadded:: 2.11

        True if `expr` correctly initializes a duration.
        Otherwise false.

            >>> Duration.is_token('8.')
            True

        Return boolean.
        '''
        try:
            Duration.__new__(Duration, expr)
            return True
        except:
            return False

    def with_denominator(self, denominator):
        '''Duration with `denominator`::

            >>> duration = Duration(1, 4)

        ::

            >>> for denominator in (4, 8, 16, 32):
            ...     print duration.with_denominator(denominator)
            ... 
            1/4
            2/8
            4/16
            8/32

        Return new duration.
        '''
        nonreduced_fraction = mathtools.NonreducedFraction(self)
        return nonreduced_fraction.with_denominator(denominator)



from abjad import *


def test_Timespan_starts_before_timespan_stops_01():
    a = timespantools.Timespan(0, 15)
    b = timespantools.Timespan(-10, -5)
    assert not a.starts_before_timespan_stops(b)

def test_Timespan_starts_before_timespan_stops_02():
    a = timespantools.Timespan(0, 15)
    b = timespantools.Timespan(-10, 0)
    assert not a.starts_before_timespan_stops(b)

def test_Timespan_starts_before_timespan_stops_03():
    a = timespantools.Timespan(0, 15)
    b = timespantools.Timespan(-10, 5)
    assert a.starts_before_timespan_stops(b)

def test_Timespan_starts_before_timespan_stops_04():
    a = timespantools.Timespan(0, 15)
    b = timespantools.Timespan(-10, 15)
    assert a.starts_before_timespan_stops(b)

def test_Timespan_starts_before_timespan_stops_05():
    a = timespantools.Timespan(0, 15)
    b = timespantools.Timespan(-10, 25)
    assert a.starts_before_timespan_stops(b)

def test_Timespan_starts_before_timespan_stops_06():
    a = timespantools.Timespan(0, 15)
    b = timespantools.Timespan(0, 10)
    assert a.starts_before_timespan_stops(b)

def test_Timespan_starts_before_timespan_stops_07():
    a = timespantools.Timespan(0, 15)
    b = timespantools.Timespan(0, 15)
    assert a.starts_before_timespan_stops(b)

def test_Timespan_starts_before_timespan_stops_08():
    a = timespantools.Timespan(0, 15)
    b = timespantools.Timespan(5, 10)
    assert a.starts_before_timespan_stops(b)

def test_Timespan_starts_before_timespan_stops_09():
    a = timespantools.Timespan(0, 15)
    b = timespantools.Timespan(5, 15)
    assert a.starts_before_timespan_stops(b)

def test_Timespan_starts_before_timespan_stops_10():
    a = timespantools.Timespan(0, 15)
    b = timespantools.Timespan(0, 25)
    assert a.starts_before_timespan_stops(b)

def test_Timespan_starts_before_timespan_stops_11():
    a = timespantools.Timespan(0, 15)
    b = timespantools.Timespan(5, 25)
    assert a.starts_before_timespan_stops(b)

def test_Timespan_starts_before_timespan_stops_12():
    a = timespantools.Timespan(0, 15)
    b = timespantools.Timespan(15, 25)
    assert a.starts_before_timespan_stops(b)

def test_Timespan_starts_before_timespan_stops_13():
    a = timespantools.Timespan(0, 15)
    b = timespantools.Timespan(20, 25)
    assert a.starts_before_timespan_stops(b)





def format_scheme_value(value, force_quotes=False):
    '''Format `value` as Scheme would:

    ::
        
        >>> schemetools.format_scheme_value(1)
        '1'
    
    ::

        >>> schemetools.format_scheme_value('foo')
        'foo'

    ::

        >>> schemetools.format_scheme_value('bar baz')
        '"bar baz"'

    ::

        >>> schemetools.format_scheme_value([1.5, True, False])
        '(1.5 #t #f)'

    Strings without whitespace can be forcibly quoted via the `force_quotes` keyword:

    ::

        >>> schemetools.format_scheme_value('foo', force_quotes=True)
        '"foo"'

    Return string.
    '''
    from abjad.tools import schemetools

    if isinstance(value, str):
        value = value.replace('"', r'\"')
        if -1 == value.find(' ') and not force_quotes:
            return value
        return '"{}"'.format(value)

    elif isinstance(value, bool):
        if value:
            return '#t'
        return '#f'

    elif isinstance(value, (list, tuple)):
        return '(%s)' % ' '.join([format_scheme_value(x) for x in value])

    elif isinstance(value, schemetools.Scheme):
        return str(value)

    elif isinstance(value, type(None)):
        return '#f'

    return str(value)



def klass_to_tools_package_qualified_klass_name(klass):
    r'''.. versionadded:: 2.10

    Change `klass` to tools package-qualified class name::

        >>> introspectiontools.klass_to_tools_package_qualified_klass_name(Note)
        'notetools.Note'

    Return string.
    '''

    module_parts = klass.__module__.split('.')
    tools_package_qualified_class_name = '.'.join(module_parts[-3:-1])

    return tools_package_qualified_class_name



from abjad.tools import componenttools


def all_are_scores(expr):
    '''.. versionadded:: 2.6

    True when `expr` is a sequence of Abjad scores::

        >>> score = Score([Staff([Note("c'4")])])

    ::

        >>> score
        Score<<1>>

    ::

        >>> scoretools.all_are_scores([score])
        True

    True when `expr` is an empty sequence::

        >>> scoretools.all_are_scores([])
        True

    Otherwise false::

        >>> scoretools.all_are_scores('foo')
        False

    Return boolean.

    Function wraps ``componenttools.all_are_components()``.
    '''
    from abjad.tools import scoretools

    return componenttools.all_are_components(expr, klasses=(scoretools.Score,))



from abjad import *


def test_RhythmTreeContainer_pop_01():

    leaf_a = rhythmtreetools.RhythmTreeLeaf(duration=3)
    leaf_b = rhythmtreetools.RhythmTreeLeaf(duration=3)
    leaf_c = rhythmtreetools.RhythmTreeLeaf(duration=2)

    container = rhythmtreetools.RhythmTreeContainer(
        duration=1, children=[leaf_a, leaf_b, leaf_c])
    assert container.children == (leaf_a, leaf_b, leaf_c)
    assert leaf_a.parent is container
    assert leaf_b.parent is container
    assert leaf_c.parent is container

    result = container.pop()
    assert container.children == (leaf_a, leaf_b)
    assert result is leaf_c
    assert result.parent is None

    result = container.pop(0)
    assert container.children == (leaf_b,)
    assert result is leaf_a
    assert result.parent is None



import abc

from abjad.tools.abctools import AttributeEqualityAbjadObject


class PitchClassObject(AttributeEqualityAbjadObject):
    '''.. versionadded:: 2.0

    Pitch-class base class.
    '''

    ### CLASS ATTRIBUTES ###

    __metaclass__ = abc.ABCMeta

    __slots__ = ()

    ### INNITIALIZER ###

    @abc.abstractmethod
    def __init__(self):
        pass

    ### SPECIAL METHODS ###

    def __hash__(self):
        return hash(repr(self))



from abjad.tools import componenttools
from abjad.tools import leaftools
from abjad.tools import sequencetools
from abjad.tools import tietools


def establish_metrical_hierarchy(components, metrical_hierarchy,
    boundary_depth=None, maximum_dot_count=None):
    r'''.. versionadded:: 2.11 

    Rewrite the contents of tie chains in an expression to match a metrical
    hierarchy.

    Example 1. Rewrite the contents of a measure in a staff using the default metrical
    hierarchy for that measure's time signature:

    ::

        >>> parseable = "abj: | 2/4 c'2 ~ || 4/4 c'32 d'2.. ~ d'16 e'32 ~ || 2/4 e'2 |"

    ::

        >>> staff = Staff(parseable)
        >>> f(staff)
        \new Staff {
            {
                \time 2/4
                c'2 ~
            }
            {
                \time 4/4
                c'32
                d'2.. ~
                d'16
                e'32 ~
            }
            {
                \time 2/4
                e'2
            }
        }

    ::

        >>> show(staff) # doctest: +SKIP

    ::

        >>> hierarchy = timesignaturetools.MetricalHierarchy((4, 4))
        >>> print hierarchy.pretty_rtm_format
        (4/4 (
            1/4
            1/4
            1/4
            1/4))

    ::

        >>> timesignaturetools.establish_metrical_hierarchy(staff[1][:], hierarchy)
        >>> f(staff)
        \new Staff {
            {
                \time 2/4
                c'2 ~
            }
            {
                \time 4/4
                c'32
                d'8.. ~
                d'2 ~
                d'8..
                e'32 ~
            }
            {
                \time 2/4
                e'2
            }
        }

    ::

        >>> show(staff) # doctest: +SKIP

    Example 2. Rewrite the contents of a measure in a staff using a custom
    metrical hierarchy:

    ::

        >>> staff = Staff(parseable)
        >>> f(staff)
        \new Staff {
            {
                \time 2/4
                c'2 ~
            }
            {
                \time 4/4
                c'32
                d'2.. ~
                d'16
                e'32 ~
            }
            {
                \time 2/4
                e'2
            }
        }

    ::

        >>> show(staff) # doctest: +SKIP

    ::

        >>> rtm = '(4/4 ((2/4 (1/4 1/4)) (2/4 (1/4 1/4))))'
        >>> hierarchy = timesignaturetools.MetricalHierarchy(rtm)
        >>> print hierarchy.pretty_rtm_format
        (4/4 (
            (2/4 (
                1/4
                1/4))
            (2/4 (
                1/4
                1/4))))

    ::

        >>> timesignaturetools.establish_metrical_hierarchy(staff[1][:], hierarchy)
        >>> f(staff)
        \new Staff {
            {
                \time 2/4
                c'2 ~
            }
            {
                \time 4/4
                c'32
                d'4... ~
                d'4...
                e'32 ~
            }
            {
                \time 2/4
                e'2
            }
        }

    ::

        >>> show(staff) # doctest: +SKIP

    Example 3. Limit the maximum number of dots per leaf using
    `maximum_dot_count`:

    ::

        >>> parseable = "abj: | 3/4 c'32 d'8 e'8 fs'4... |"
        >>> measure = p(parseable)
        >>> f(measure)
        {
            \time 3/4
            c'32
            d'8
            e'8
            fs'4...
        }

    ::

        >>> show(measure) # doctest: +SKIP

    Without constraining the `maximum_dot_count`:

    ::

        >>> timesignaturetools.establish_metrical_hierarchy(measure[:], measure)
        >>> f(measure)
        {
            \time 3/4
            c'32
            d'16. ~
            d'32
            e'16. ~
            e'32
            fs'4...
        }

    ::

        >>> show(measure) # doctest: +SKIP
        
    Constraining the `maximum_dot_count` to `2`:

    ::

        >>> measure = p(parseable)
        >>> timesignaturetools.establish_metrical_hierarchy(measure[:], measure,
        ...     maximum_dot_count=2)
        >>> f(measure)
        {
            \time 3/4
            c'32
            d'16. ~
            d'32
            e'16. ~
            e'32
            fs'8.. ~
            fs'4
        }

    ::

        >>> show(measure) # doctest: +SKIP

    Constraining the `maximum_dot_count` to `1`:

    ::

        >>> measure = p(parseable)
        >>> timesignaturetools.establish_metrical_hierarchy(measure[:], measure,
        ...     maximum_dot_count=1)
        >>> f(measure)
        {
            \time 3/4
            c'32
            d'16. ~
            d'32
            e'16. ~
            e'32
            fs'16. ~
            fs'8 ~
            fs'4
        }

    ::

        >>> show(measure) # doctest: +SKIP

    Constraining the `maximum_dot_count` to `0`:

    ::

        >>> measure = p(parseable)
        >>> timesignaturetools.establish_metrical_hierarchy(measure[:], measure,
        ...     maximum_dot_count=0)
        >>> f(measure)
        {
            \time 3/4
            c'32
            d'32 ~
            d'16 ~
            d'32
            e'32 ~
            e'16 ~
            e'32
            fs'32 ~
            fs'16 ~
            fs'8 ~
            fs'4
        }

    ::

        >>> show(measure) # doctest: +SKIP

    Example 4: Split tie chains at different depths of the `MetricalHierarchy`,
    if those tie chains cross any offsets at that depth, but do not also both
    begin and end at any of those offsets.

    Consider the default metrical hierarchy for `9/8`:

        >>> hierarchy = timesignaturetools.MetricalHierarchy((9, 8))
        >>> print hierarchy.pretty_rtm_format
        (9/8 (
            (3/8 (
                1/8
                1/8
                1/8))
            (3/8 (
                1/8
                1/8
                1/8))
            (3/8 (
                1/8
                1/8
                1/8))))

    We can establish that hierarchy without specifying a `boundary_depth`:

    ::

        >>> parseable = "abj: | 9/8 c'2 d'2 e'8 |"
        >>> measure = p(parseable)
        >>> f(measure)
        {
            \time 9/8
            c'2
            d'2
            e'8
        }

    ::

        >>> show(measure) # doctest: +SKIP

    ::

        >>> timesignaturetools.establish_metrical_hierarchy(measure[:], measure)
        >>> f(measure)
        {
            \time 9/8
            c'2
            d'4 ~
            d'4
            e'8
        }

    ::

        >>> show(measure) # doctest: +SKIP

    With a `boundary_depth` of `1`, tie chains which cross any offsets created
    by nodes with a depth of `1` in this MetricalHierarchy's rhythm tree - i.e.
    `0/8`, `3/8`, `6/8` and `9/8` - which do not also begin and end at any of those
    offsets, will be split:

    ::

        >>> measure = p(parseable)
        >>> timesignaturetools.establish_metrical_hierarchy(measure[:], measure,
        ...     boundary_depth=1)
        >>> f(measure)
        {
            \time 9/8
            c'4. ~
            c'8
            d'4 ~
            d'4
            e'8
        }

    ::

        >>> show(measure) # doctest: +SKIP

    For this `9/8` hierarchy, and this input notation, A `boundary_depth` of `2`
    causes no change, as all tie chains already align to multiples of `1/8`:

    ::

        >>> measure = p(parseable)
        >>> timesignaturetools.establish_metrical_hierarchy(measure[:], measure,
        ...     boundary_depth=2)
        >>> f(measure)
        {
            \time 9/8
            c'2
            d'4 ~
            d'4
            e'8
        }

    ::

        >>> show(measure) # doctest: +SKIP

    Example 5. Comparison of `3/4` and `6/8`, at `boundary_depths` of 0 and 1:

    ::

        >>> triple = "abj: | 3/4 2 4 || 3/4 4 2 || 3/4 4. 4. |"
        >>> triple += "| 3/4 2 ~ 8 8 || 3/4 8 8 ~ 2 |"
        >>> duples = "abj: | 6/8 2 4 || 6/8 4 2 || 6/8 4. 4. |"
        >>> duples += "| 6/8 2 ~ 8 8 || 6/8 8 8 ~ 2 |"
        >>> score = Score([Staff(triple), Staff(duples)])

    In order to see the different time signatures on each staff, we need to
    move some engravers from the Score context to the Staff context:

    ::

        >>> engravers = ['Timing_translator', 'Time_signature_engraver',
        ...     'Default_bar_line_engraver']
        >>> score.engraver_removals.extend(engravers)
        >>> score[0].engraver_consists.extend(engravers)
        >>> score[1].engraver_consists.extend(engravers)
        >>> f(score)
        \new Score \with {
            \remove Timing_translator
            \remove Time_signature_engraver
            \remove Default_bar_line_engraver
        } <<
            \new Staff \with {
                \consists Timing_translator
                \consists Time_signature_engraver
                \consists Default_bar_line_engraver
            } {
                {
                    \time 3/4
                    c'2
                    c'4
                }
                {
                    c'4
                    c'2
                }
                {
                    c'4.
                    c'4.
                }
                {
                    c'2 ~
                    c'8
                    c'8
                }
                {
                    c'8
                    c'8 ~
                    c'2
                }
            }
            \new Staff \with {
                \consists Timing_translator
                \consists Time_signature_engraver
                \consists Default_bar_line_engraver
            } {
                {
                    \time 6/8
                    c'2
                    c'4
                }
                {
                    c'4
                    c'2
                }
                {
                    c'4.
                    c'4.
                }
                {
                    c'2 ~
                    c'8
                    c'8
                }
                {
                    c'8
                    c'8 ~
                    c'2
                }
            }
        >>

    ::

        >>> show(score) # doctest: +SKIP

    Here we establish a metrical hierarchy without specifying and boundary depth:

    ::

        >>> for measure in iterationtools.iterate_measures_in_expr(score):
        ...     timesignaturetools.establish_metrical_hierarchy(measure[:], measure)
        >>> f(score)
        \new Score \with {
            \remove Timing_translator
            \remove Time_signature_engraver
            \remove Default_bar_line_engraver
        } <<
            \new Staff \with {
                \consists Timing_translator
                \consists Time_signature_engraver
                \consists Default_bar_line_engraver
            } {
                {
                    \time 3/4
                    c'2
                    c'4
                }
                {
                    c'4
                    c'2
                }
                {
                    c'4.
                    c'4.
                }
                {
                    c'2 ~
                    c'8
                    c'8
                }
                {
                    c'8
                    c'8 ~
                    c'2
                }
            }
            \new Staff \with {
                \consists Timing_translator
                \consists Time_signature_engraver
                \consists Default_bar_line_engraver
            } {
                {
                    \time 6/8
                    c'2
                    c'4
                }
                {
                    c'4
                    c'2
                }
                {
                    c'4.
                    c'4.
                }
                {
                    c'4. ~
                    c'4
                    c'8
                }
                {
                    c'8
                    c'4 ~
                    c'4.
                }
            }
        >>

    ::

        >>> show(score) # doctest: +SKIP

    Here we re-establish metrical hierarchy at a boundary depth of `1`:

    ::

        >>> for measure in iterationtools.iterate_measures_in_expr(score):
        ...     timesignaturetools.establish_metrical_hierarchy(
        ...         measure[:], measure, boundary_depth=1)
        ...
        >>> f(score)
        \new Score \with {
            \remove Timing_translator
            \remove Time_signature_engraver
            \remove Default_bar_line_engraver
        } <<
            \new Staff \with {
                \consists Timing_translator
                \consists Time_signature_engraver
                \consists Default_bar_line_engraver
            } {
                {
                    \time 3/4
                    c'2
                    c'4
                }
                {
                    c'4
                    c'2
                }
                {
                    c'4 ~
                    c'8
                    c'8 ~
                    c'4
                }
                {
                    c'2 ~
                    c'8
                    c'8
                }
                {
                    c'8
                    c'8 ~
                    c'2
                }
            }
            \new Staff \with {
                \consists Timing_translator
                \consists Time_signature_engraver
                \consists Default_bar_line_engraver
            } {
                {
                    \time 6/8
                    c'4. ~
                    c'8
                    c'4
                }
                {
                    c'4
                    c'8 ~
                    c'4.
                }
                {
                    c'4.
                    c'4.
                }
                {
                    c'4. ~
                    c'4
                    c'8
                }
                {
                    c'8
                    c'4 ~
                    c'4.
                }
            }
        >>

    ::

        >>> show(score) # doctest: +SKIP

    Note that the two time signatures are much more clearly disambiguated above.

    Example 6. Establishing metrical hierarchy recursively in measures with
    nested tuplets:

    ::

        >>> measure = p("abj: | 4/4 c'16 ~ c'4 d'8. ~ " \
        ...     "2/3 { d'8. ~ 3/5 { d'16 e'8. f'16 ~ } } f'4 |")
        >>> f(measure)
        {
            \time 4/4
            c'16 ~
            c'4
            d'8. ~
            \times 2/3 {
                d'8. ~
                \fraction \times 3/5 {
                    d'16
                    e'8.
                    f'16 ~
                }
            }
            f'4
        }

    ::

        >>> show(measure) # doctest: +SKIP

    When establishing a metrical hierarchy on a selection of components which
    contain containers, like `Tuplets` or `Containers`,
    `timesignaturetools.establish_metrical_hierarchy()` will recurse into
    those containers, treating them as measures whose time signature is derived
    from the preprolated duration of the container's contents:

    ::

        >>> timesignaturetools.establish_metrical_hierarchy(measure[:], measure,
        ...     boundary_depth=1)
        >>> f(measure)
        {
            \time 4/4
            c'4 ~
            c'16
            d'8. ~
            \times 2/3 {
                d'8 ~
                d'16 ~
                \fraction \times 3/5 {
                    d'16
                    e'8 ~
                    e'16
                    f'16 ~
                }
            }
            f'4
        }

    ::

        >>> show(measure) # doctest: +SKIP    

    Operate in place and return none.
    '''

    from abjad.tools import timesignaturetools

    def get_offsets_at_depth(depth):
        if depth < len(offset_inventory):
            return offset_inventory[depth]
        while len(offset_inventory) <= depth: 
            new_offsets = []
            old_offsets = offset_inventory[-1]
            for first, second in sequencetools.iterate_sequence_pairwise_strict(old_offsets):
                new_offsets.append(first)
                new_offsets.append((first + second) / 2)
            new_offsets.append(old_offsets[-1])
            offset_inventory.append(tuple(new_offsets))
        return offset_inventory[depth]

    def is_acceptable_tie_chain(tie_chain_duration, 
        tie_chain_starts_in_offsets,
        tie_chain_stops_in_offsets):
        #print '\tTESTING ACCEPTABILITY'
        if not tie_chain_duration.is_assignable:
            return False
        if maximum_dot_count is not None and \
            maximum_dot_count < tie_chain_duration.dot_count:
            return False
        if not tie_chain_starts_in_offsets and not tie_chain_stops_in_offsets:
            return False
        return True

    def is_boundary_crossing_tie_chain(tie_chain_start_offset, tie_chain_stop_offset):
        #print '\tTESTING BOUNDARY CROSSINGS'
        if boundary_depth is None:
            return False
        if not any(tie_chain_start_offset < x < tie_chain_stop_offset for x in boundary_offsets):
            return False
        if tie_chain_start_offset in boundary_offsets and \
            tie_chain_stop_offset in boundary_offsets:
            return False
        return True

    def recurse(tie_chain, depth=0):
        offsets = get_offsets_at_depth(depth)
        #print 'DEPTH:', depth

        tie_chain_duration = tie_chain.preprolated_duration
        tie_chain_start_offset = tie_chain.start_offset
        tie_chain_stop_offset = tie_chain.stop_offset
        tie_chain_starts_in_offsets = tie_chain_start_offset in offsets
        tie_chain_stops_in_offsets = tie_chain_stop_offset in offsets     

        if not is_acceptable_tie_chain(
            tie_chain_duration,
            tie_chain_starts_in_offsets, 
            tie_chain_stops_in_offsets):

            #print 'UNACCEPTABLE:', tie_chain, tie_chain_start_offset, tie_chain_stop_offset
            #print '\t', ' '.join([str(x) for x in offsets])
            split_offset = None
            offsets = get_offsets_at_depth(depth)

            # if the tie chain's start aligns, take the latest possible offset
            if tie_chain_starts_in_offsets:
                offsets = reversed(offsets)

            for offset in offsets:
                if tie_chain_start_offset < offset < tie_chain_stop_offset:
                    split_offset = offset
                    break

            #print '\tABS:', split_offset
            if split_offset is not None:
                split_offset -= tie_chain_start_offset
                #print '\tREL:', split_offset
                #print ''
                tie_chains = [tietools.TieChain(shard) for shard in
                    componenttools.split_components_at_offsets(tie_chain[:], [split_offset])]
                for tie_chain in tie_chains:
                    recurse(tie_chain, depth=depth)
            else:
                #print ''
                recurse(tie_chain, depth=depth+1)

        elif is_boundary_crossing_tie_chain(
            tie_chain_start_offset,
            tie_chain_stop_offset):

            #print 'BOUNDARY CROSSING', tie_chain, tie_chain_start_offset, tie_chain_stop_offset
            offsets = boundary_offsets
            if tie_chain_start_offset in boundary_offsets:
                offsets = reversed(boundary_offsets)
            split_offset = None
            for offset in offsets:
                if tie_chain_start_offset < offset < tie_chain_stop_offset:
                    split_offset = offset
                    break
            assert split_offset is not None 
            #print '\tABS:', split_offset
            split_offset -= tie_chain_start_offset
            #print '\tREL:', split_offset
            #print ''
            tie_chains = [tietools.TieChain(shard) for shard in
                componenttools.split_components_at_offsets(tie_chain[:], [split_offset])]
            for tie_chain in tie_chains:
                recurse(tie_chain, depth=depth)

        else:
            #print 'ACCEPTABLE:', tie_chain, tie_chain_start_offset, tie_chain_stop_offset
            #print '\t', ' '.join([str(x) for x in offsets])
            #print ''
            leaftools.fuse_leaves(tie_chain[:])

    # check arguments
    assert componenttools.all_are_thread_contiguous_components(components)
    metrical_hierarchy = timesignaturetools.MetricalHierarchy(metrical_hierarchy)
    assert sum([x.preprolated_duration for x in components]) == metrical_hierarchy.duration
    if boundary_depth is not None:
        boundary_depth = int(boundary_depth)
    if maximum_dot_count is not None:
        maximum_dot_count = int(maximum_dot_count)
        assert 0 <= maximum_dot_count

    # build offset inventory, adjusted for initial offset and prolation
    first_offset = components[0].start_offset
    prolation = components[0].prolation
    offset_inventory= []
    for offsets in metrical_hierarchy.depthwise_offset_inventory:
        offsets = [(x * prolation) + first_offset for x in offsets]
        offset_inventory.append(tuple(offsets))

    # build boundary offset inventory, if applicable
    if boundary_depth is not None:
        boundary_offsets = offset_inventory[boundary_depth]

    # cache results of iterator, as we'll be mutating the underlying collection
    items = tuple(tietools.iterate_topmost_masked_tie_chains_and_containers_in_expr(components))
    for item in items:
        if isinstance(item, tietools.TieChain):
            #print 'RECURSING:', item
            recurse(item, depth=0)
        else:
            #print 'DESCENDING:', item
            duration = sum([x.preprolated_duration for x in item])
            sub_metrical_hierarchy = timesignaturetools.MetricalHierarchy(duration)
            sub_boundary_depth = 1
            if boundary_depth is None:
                sub_boundary_depth = None
            establish_metrical_hierarchy(item[:], sub_metrical_hierarchy,
                boundary_depth=sub_boundary_depth, maximum_dot_count=maximum_dot_count)



from abjad.tools.formattools.get_context_mark_format_pieces import get_context_mark_format_pieces
from abjad.tools.formattools.is_formattable_context_mark_for_component import is_formattable_context_mark_for_component


def get_context_mark_format_contributions_for_slot(component, slot):
    r'''.. versionadded:: 2.0

    Get context mark format contributions for `component` at `slot`.

    Return list.
    '''
    from abjad.tools import componenttools
    from abjad.tools import contexttools
    from abjad.tools import measuretools

    result = []
    marks = set([])
    candidates = contexttools.get_context_marks_attached_to_any_improper_parent_of_component(
        component)

    for candidate in candidates:
        if candidate._format_slot == slot:
            if is_formattable_context_mark_for_component(candidate, component):
                marks.add(candidate)

    for mark in marks:
        result.extend(get_context_mark_format_pieces(mark))

    result.sort()
    return ['context marks', result]



from abjad import *


def test_DuratedComplexBeamSpanner___copy___01():

    staff = Staff([
        Container("c'32 d'32 e'32"),
        Container("f'32 g'32 a'32"),
        Container("b'32 c'32")
        ])
    durations = [x.preprolated_duration for x in staff]
    beam = beamtools.DuratedComplexBeamSpanner(staff[:], durations=durations, span=2, direction=Down)

    new_staff = componenttools.copy_components_and_covered_spanners([staff])[0]
    new_beam = beamtools.get_beam_spanner_attached_to_component(new_staff[0])

    assert staff.lilypond_format == new_staff.lilypond_format
    assert new_beam.durations == beam.durations
    assert new_beam.span == beam.span
    assert new_beam.lone == beam.lone
    assert new_beam.direction == beam.direction



from abjad.tools.documentationtools.GraphvizGraph import GraphvizGraph


class GraphvizSubgraph(GraphvizGraph):
    '''A Graphviz cluster subgraph.'''

    ### INITIALIZER ###

    def __init__(self,
        attributes=None,
        children=None,
        edge_attributes=None,
        is_cluster=True,
        name=None,
        node_attributes=None
        ):
        GraphvizGraph.__init__(self,
            attributes=attributes,
            children=children,
            edge_attributes=edge_attributes,
            name=name,
            node_attributes=node_attributes
            )
        self.is_cluster = is_cluster
        self._edges = set([])

    ### READ-ONLY PRIVATE PROPERTIES ###

    @property
    def _graphviz_format_pieces(self):
        result = []
        return result

    @property
    def _node_klass(self):
        from abjad.tools import documentationtools
        return (documentationtools.GraphvizSubgraph, documentationtools.GraphvizNode)

    ### READ-ONLY PUBLIC PROPERTIES ###
 
    @property
    def canonical_name(self):
        if self.name is not None:
            if self.is_cluster:
                return 'cluster_' + self.name
            return self.name
        prefix = 'subgraph_'
        if self.is_cluster:
            prefix = 'cluster_'
        return prefix + '_'.join(str(x) for x in self.graph_order)

    @property
    def edges(self):
        return tuple(self._edges)

    ### READ/WRITE PUBLIC PROPERTIES ###

    @apply
    def is_cluster():
        def fget(self):
            return self._is_cluster
        def fset(self, arg):
            self._is_cluster = bool(arg)
        return property(**locals())




from abjad.tools.documentationtools.GraphvizObject import GraphvizObject


class GraphvizEdge(GraphvizObject):
    '''A Graphviz edge.'''

    ### INITIALIZER ###

    def __init__(self, attributes=None, is_directed=True):
        GraphvizObject.__init__(self, attributes=attributes)
        self._head = None
        self._tail = None
        self._is_directed = bool(is_directed)

    ### SPECIAL METHODS ###

    def __call__(self, *args):
        from abjad.tools import documentationtools
        if args:
            assert len(args) == 2
            assert all(isinstance(x,
                (documentationtools.GraphvizSubgraph, documentationtools.GraphvizNode))
                for x in args)
            tail, head = args
            self._disconnect()
            self._connect(tail, head)
        else:
            self._disconnect()
        return self

    ### PRIVATE METHODS ###

    def _connect(self, tail, head):
        tail._edges.add(self)
        head._edges.add(self)
        self._tail = tail
        self._head = head

    def _disconnect(self):
        if self.tail is not None:
            self.tail._edges.remove(self)
        if self.head is not None:
            self.head._edges.remove(self)
        self._tail = None
        self._head = None

    ### READ-ONLY PRIVATE PROPERTIES ###

    @property
    def _graphviz_format_contributions(self):
        connection = '->'
        if not self.is_directed:
            connection = '--'
        edge_def = '{} {} {}'.format(
            self._format_value(self.tail.canonical_name),
            connection,
            self._format_value(self.head.canonical_name))
        if len(self.attributes):
            result = self._format_attribute_list(self.attributes)
            result[0] = '{} {}'.format(edge_def, result[0])
            return result
        return [edge_def + ';']

    ### READ-ONLY PUBLIC PROPERTIES ###

    @property
    def head(self):
        return self._head

    @property
    def tail(self):
        return self._tail

    ### READ/WRITE PUBLIC PROPERTIES ###

    @apply
    def is_directed():
        def fget(self):
            return self._is_directed
        def fset(self, arg):
            self._is_directed = bool(arg)
        return property(**locals())




from abjad.tools.beamtools.BeamSpanner import BeamSpanner


class MultipartBeamSpanner(BeamSpanner):
    r'''.. versionadded:: 2.0

    Abjad multipart beam spanner::

        >>> staff = Staff("c'8 d'8 e'4 f'8 g'8 r4")

    ::

        >>> show(staff) # doctest: +SKIP

    ::

        >>> beamtools.MultipartBeamSpanner(staff[:])
        MultipartBeamSpanner(c'8, d'8, e'4, f'8, g'8, r4)

    ::

        >>> f(staff)
        \new Staff {
            c'8 [
            d'8 ]
            e'4
            f'8 [
            g'8 ]
            r4
        }

    ::

        >>> show(staff) # doctest: +SKIP

    Avoid rests.

    Avoid large-duration notes.

    Return multipart beam spanner.
    '''

    ### INITIALIZER ###

    def __init__(self, components=None, direction=None):
        BeamSpanner.__init__(self, components, direction=direction)

    ### PRIVATE METHODS ###

    def _format_right_of_leaf(self, leaf):
        from abjad.tools import beamtools
        from abjad.tools import leaftools
        result = []
        direction_string = ''
        if self.direction is not None:
            direction_string = '%s ' % self.direction
        if beamtools.is_beamable_component(leaf):
            if 1 < len(self.leaves):
                prev = leaftools.get_nth_leaf_in_thread_from_leaf(leaf, -1)
                if id(prev) not in [id(x) for x in self.leaves]:
                    prev = None
                next = leaftools.get_nth_leaf_in_thread_from_leaf(leaf, 1)
                if id(next) not in [id(x) for x in self.leaves]:
                    next = None
                if self._is_my_first_leaf(leaf):
                    if next is not None:
                        if beamtools.is_beamable_component(next):
                            result.append('%s[' % direction_string)
                else:
                    if prev is not None:
                        if not beamtools.is_beamable_component(prev):
                            if next is not None:
                                result.append('%s[' % direction_string)
                if self._is_my_last_leaf(leaf):
                    if prev is not None:
                        if beamtools.is_beamable_component(prev):
                            result.append(']')
                else:
                    next = leaftools.get_nth_leaf_in_thread_from_leaf(leaf, 1)
                    if next is not None and not beamtools.is_beamable_component(next):
                        result.append(']')
        return result



import platform

_python_version = platform.python_version()
if not _python_version.startswith('2.7'):
    print 'WARNING: Abjad no longer supports versions of Python less than 2.7!'
    print 'WARNING: Please upgrade your version of Python to 2.7!'

import readline
import rlcompleter
if 'libedit' in readline.__doc__:
    readline.parse_and_bind("bind ^I rl_complete")
else:
    readline.parse_and_bind("tab: complete")

# setup .abjad directory and friends (if not already handled elsewhere)
# configurationtools.setup_abjad_directory_silently()
from abjad.tools.configurationtools import AbjadConfig
ABJCFG = AbjadConfig()
del AbjadConfig

from abjad.tools.importtools._import_functions_in_package_to_namespace import \
    _import_functions_in_package_to_namespace
from abjad.tools import *
from abjad.tools.chordtools import Chord
from abjad.tools.containertools import Container
from abjad.tools.durationtools import Duration
from abjad.tools.durationtools import Multiplier
from abjad.tools.durationtools import Offset
from abjad.tools.iotools import f
from abjad.tools.iotools import p
from abjad.tools.iotools import play
from abjad.tools.iotools import show
from abjad.tools.iotools import z
from abjad.tools.measuretools import Measure
from abjad.tools.notetools import Note
from abjad.tools.resttools import Rest
from abjad.tools.scoretools import Score
from abjad.tools.stafftools import Staff
from abjad.tools.tuplettools import Tuplet
from abjad.tools.voicetools import Voice
from fractions import Fraction
import os

# import Abjad exceptions in __builtins__ namespace
_exceptions_path = os.path.join(__path__[0], 'tools', 'exceptiontools')
_import_functions_in_package_to_namespace(_exceptions_path, __builtins__)

del os
del platform
del readline
del rlcompleter
del tools

__version__ = '2.11'



import numbers
from abjad.tools import durationtools


def make_rests(durations, decrease_durations_monotonically=True, tie_parts=False):
    r'''.. versionadded:: 1.1

    Make rests.

    Make rests and drecrease durations monotonically::

        >>> resttools.make_rests([(5, 16), (9, 16)], decrease_durations_monotonically=True)
        [Rest('r4'), Rest('r16'), Rest('r2'), Rest('r16')]

    Makes rests and increase durations monotonically::

        >>> resttools.make_rests([(5, 16), (9, 16)], decrease_durations_monotonically=False)
        [Rest('r16'), Rest('r4'), Rest('r16'), Rest('r2')]

    Make tied rests::

        >>> voice = Voice(resttools.make_rests([(5, 16), (9, 16)], tie_parts=True))

    ::

        >>> f(voice)
        \new Voice {
            r4 ~
            r16
            r2 ~
            r16
        }

    Return list of rests.

    .. versionchanged:: 2.0
        renamed ``construct.rests()`` to
        ``resttools.make_rests()``.
    '''
    from abjad.tools import resttools

    # check input
    if isinstance(durations, (numbers.Number, tuple)):
        durations = [durations]

    # make rests
    result = []
    for duration in durations:
        result.extend(resttools.make_tied_rest(
            duration, decrease_durations_monotonically=decrease_durations_monotonically, 
            tie_parts=tie_parts))

    # return result
    return result



import copy
from abjad.tools import sequencetools
from abjad.tools.abctools import AbjadObject


class TreeNode(AbjadObject):
    '''A node in a generalized tree.

    Return `TreeNode` instance.
    '''

    ### INITIALIZER ###

    def __init__(self, name=None):
        self._name = None
        self._parent = None    
        self.name = name

    ### SPECIAL METHODS ###

    def __copy__(self, *args):
        return type(self)(
            *[copy.deepcopy(x) for x in self.__getnewargs__()]
            )

    __deepcopy__ = __copy__

    def __eq__(self, other):
        if type(self) == type(other):
            return True
        return False

    def __getnewargs__(self):
        return self._positional_argument_values + self._keyword_argument_values

    def __getstate__(self):
        state = {}
        #for name in self._positional_argument_names:
        #    state['_' + name] = getattr(self, name)
        #for name in self._keyword_argument_names:
        #    state['_' + name] = getattr(self, name)
        #state['_parent'] = self._parent
        class_dir = set(dir(self.__class__))
        self_dir = set(x for x in dir(self) if x.startswith('_') and not x.startswith('__'))
        for name in self_dir.difference(class_dir):
            state[name] = getattr(self, name)
        return state

    def __ne__(self, other):
        return not self.__eq__(other)

    def __repr__(self):

        def format_mapping(name, value):
            if not value:
                return None
            result = ['\t{}={{'.format(name)]
            items = sorted(value.items())
            if 1 < len(items):
                for item in items[:-1]:
                    result.append('\t\t{!r}: {!r},'.format(*item))
            result.append('\t\t{!r}: {!r}'.format(*items[-1]))
            result.append('\t\t}')
            return result

        def format_tuple(name, value):
            if not value:
                return None
            result=['\t{}=('.format(name)]
            items = [repr(x).splitlines() for x in value]
            if 1 < len(items):
                for item in items[:-1]:
                    result.extend('\t\t' + x for x in item[:-1])
                    result.append('\t\t' + item[-1] + ',')
                result.extend('\t\t' + x for x in items[-1])
            else:
                result.extend('\t\t' + x for x in items[0][:-1])
                result.append('\t\t' + items[0][-1] + ',')
            result.append('\t\t)')
            return result

        def format_other(name, value):
            if value is None:
                return None
            item = repr(value).splitlines()
            result = ['\t{}={}'.format(name, item[0])]
            result.extend('\t\t' + x for x in item[1:])
            return result

        attr_pieces = []
        for name in self._keyword_argument_names:
            value = getattr(self, name)
            if isinstance(value, (list, tuple)):
                attr_piece = format_tuple(name, value)
            elif isinstance(value, dict):
                attr_piece = format_mapping(name, value)
            else:
                attr_piece = format_other(name, value)
            if attr_piece is not None:
                attr_pieces.append(attr_piece)

        if not attr_pieces:
            return '{}()'.format(self._class_name)

        result = ['{}('.format(self._class_name)]
        if 1 < len(attr_pieces):
            for attr_piece in attr_pieces[:-1]:
                attr_piece[-1] += ','
                result.extend(attr_piece)
        result.extend(attr_pieces[-1]) 
        result.append('\t)')
        return '\n'.join(result)

    def __setstate__(self, state):
        for key, value in state.iteritems():
            setattr(self, key, value)

    ### PRIVATE METHODS ###

    def _get_node_state_flags(self):
        state_flags = {}
        for name in self._state_flag_names:
            state_flags[name] = True
            for node in self.improper_parentage:
                if not getattr(node, name):
                    state_flags[name] = False
                    break
        return state_flags

    def _mark_entire_tree_for_later_update(self):
        for node in self.improper_parentage:
            for name in self._state_flag_names:
                setattr(node, name, False)

    def _switch_parent(self, new_parent):

        name_dictionary = {}
        if hasattr(self, '_named_children'):
            for name, children in self._named_children.iteritems():
                name_dictionary[name] = copy.copy(children)
        if hasattr(self, 'name') and self.name is not None:
            if self.name not in name_dictionary:
                name_dictionary[self.name] = set([])
            name_dictionary[self.name].add(self)

        if self._parent is not None and name_dictionary:
            for parent in self.proper_parentage:
                named_children = parent._named_children
                for name in name_dictionary:
                    for node in name_dictionary[name]:
                        named_children[name].remove(node)
                    if not named_children[name]:
                        del named_children[name]

        if self._parent is not None:
            index = self._parent.index(self)
            self._parent._children.pop(index)
        self._parent = new_parent

        if new_parent is not None and name_dictionary:
            for parent in self.proper_parentage:
                named_children = parent._named_children
                for name in name_dictionary:
                    if name in named_children:
                        named_children[name].update(name_dictionary[name])
                    else:
                        named_children[name] = copy.copy(name_dictionary[name])

        self._mark_entire_tree_for_later_update()

    ### READ-ONLY PRIVATE PROPERTIES ###

    @property
    def _state_flag_names(self):
        return ()

    ### READ-ONLY PUBLIC PROPERTIES ###

    @property
    def depth(self):
        '''The depth of a node in a rhythm-tree structure:

        ::

            >>> a = datastructuretools.TreeContainer()
            >>> b = datastructuretools.TreeContainer()
            >>> c = datastructuretools.TreeNode()
            >>> a.append(b)
            >>> b.append(c)
        
        ::

            >>> a.depth
            0

        ::

            >>> a[0].depth
            1

        ::

            >>> a[0][0].depth
            2

        Return int.
        '''
        node = self
        depth = 0
        while node.parent is not None:
            depth += 1
            node = node.parent
        return depth

    @property
    def depthwise_inventory(self):
        '''A dictionary of all nodes in a rhythm-tree, organized by their
        depth relative the root node:

        ::

            >>> a = datastructuretools.TreeContainer(name='a')
            >>> b = datastructuretools.TreeContainer(name='b')
            >>> c = datastructuretools.TreeContainer(name='c')
            >>> d = datastructuretools.TreeContainer(name='d')
            >>> e = datastructuretools.TreeContainer(name='e')
            >>> f = datastructuretools.TreeContainer(name='f')
            >>> g = datastructuretools.TreeContainer(name='g')
            
        ::

            >>> a.extend([b, c])
            >>> b.extend([d, e])
            >>> c.extend([f, g])

        ::

            >>> inventory = a.depthwise_inventory
            >>> for depth in sorted(inventory):
            ...     print 'DEPTH: {}'.format(depth)
            ...     for node in inventory[depth]:
            ...         print node.name 
            ...
            DEPTH: 0
            a
            DEPTH: 1
            b
            c
            DEPTH: 2
            d
            e
            f
            g

        Return dictionary.
        '''
        inventory = {}
        def recurse(node):
            if node.depth not in inventory:
                inventory[node.depth] = []
            inventory[node.depth].append(node)
            if hasattr(node, 'children'):
                for child in node.children:
                    recurse(child)
        recurse(self)
        return inventory

    @property
    def graph_order(self):
        order = []
        for parent, child in sequencetools.iterate_sequence_pairwise_strict(
            reversed(self.improper_parentage)):
            order.append(parent.index(child))
        return tuple(order)

    @property
    def improper_parentage(self):
        '''The improper parentage of a node in a rhythm-tree, being the 
        sequence of node beginning with itself and ending with the root node
        of the tree:

        ::

            >>> a = datastructuretools.TreeContainer()
            >>> b = datastructuretools.TreeContainer()
            >>> c = datastructuretools.TreeNode()

        ::

            >>> a.append(b)
            >>> b.append(c)

        ::

            >>> a.improper_parentage == (a,)
            True

        ::

            >>> b.improper_parentage == (b, a)
            True

        ::

            >>> c.improper_parentage == (c, b, a)
            True

        Return tuple of `TreeNode` instances.
        '''
        node = self
        parentage = [node]
        while node.parent is not None:
            node = node.parent
            parentage.append(node)
        return tuple(parentage)

    @property
    def parent(self):
        '''The node's parent node:

        ::

            >>> a = datastructuretools.TreeContainer()
            >>> b = datastructuretools.TreeContainer()
            >>> c = datastructuretools.TreeNode()

        ::

            >>> a.append(b)
            >>> b.append(c)

        ::

            >>> a.parent is None
            True

        ::

            >>> b.parent is a
            True

        ::

            >>> c.parent is b
            True

        Return `TreeNode` instance.
        '''
        return self._parent

    @property
    def proper_parentage(self):
        '''The proper parentage of a node in a rhythm-tree, being the 
        sequence of node beginning with the node's immediate parent and
        ending with the root node of the tree:

        ::

            >>> a = datastructuretools.TreeContainer()
            >>> b = datastructuretools.TreeContainer()
            >>> c = datastructuretools.TreeNode()

        ::

            >>> a.append(b)
            >>> b.append(c)

        ::

            >>> a.proper_parentage == ()
            True

        ::

            >>> b.proper_parentage == (a,)
            True

        ::

            >>> c.proper_parentage == (b, a)
            True

        Return tuple of `TreeNode` instances.
        '''
        return self.improper_parentage[1:]

    @property
    def root(self):
        '''The root node of the tree: that node in the tree which has
        no parent:

        ::

            >>> a = datastructuretools.TreeContainer()
            >>> b = datastructuretools.TreeContainer()
            >>> c = datastructuretools.TreeNode()

        ::

            >>> a.append(b)
            >>> b.append(c)

        ::

            >>> a.root is a
            True
            >>> b.root is a
            True
            >>> c.root is a
            True

        Return `TreeNode` instance.
        '''
        node = self
        while node.parent is not None:
            node = node.parent
        return node

    ### READ/WRITE PROPERTIES ###

    @apply
    def name():
        def fget(self):
            return self._name
        def fset(self, arg):
            assert isinstance(arg, (str, type(None)))
            old_name = self._name
            for parent in self.proper_parentage:
                named_children = parent._named_children
                if old_name is not None:
                    named_children[old_name].remove(self)
                    if not named_children[old_name]:
                        del named_children[old_name]
                if arg is not None:
                    if arg not in named_children:
                        named_children[arg] = set([self])
                    else:
                        named_children[arg].add(self)
            self._name = arg
        return property(**locals())




from abjad import *


def test_Measure___delitem___01():
    '''Nonnegative indices work.
    
    Automatically update time signature.
    '''

    measure = Measure((4, 8), Note(0, (1, 8)) * 4)
    measure.automatically_adjust_time_signature = True
    del(measure[:1])

    assert wellformednesstools.is_well_formed_component(measure)
    assert measure.lilypond_format == "{\n\t\\time 3/8\n\tc'8\n\tc'8\n\tc'8\n}"


def test_Measure___delitem___02():
    '''Negative indices work.

    Automatically update time signatures.
    '''

    measure = Measure((4, 8), Note(0, (1, 8)) * 4)
    measure.automatically_adjust_time_signature = True
    del(measure[-1:])

    assert wellformednesstools.is_well_formed_component(measure)
    assert measure.lilypond_format == "{\n\t\\time 3/8\n\tc'8\n\tc'8\n\tc'8\n}"


def test_Measure___delitem___03():
    '''Denominator preservation in time signature.

    Automatically update time signature.
    '''

    measure = Measure((4, 8), Note(0, (1, 8)) * 4)
    measure.automatically_adjust_time_signature = True
    del(measure[:2])

    assert wellformednesstools.is_well_formed_component(measure)
    assert measure.lilypond_format == "{\n\t\\time 2/8\n\tc'8\n\tc'8\n}"


def test_Measure___delitem___04():
    '''Denominator changes from 8 to 16.

    Automatically update time signature.
    '''

    measure = Measure((4, 8), Note(0, (1, 16)) * 2 + Note(0, (1, 8)) * 3)
    measure.automatically_adjust_time_signature = True
    del(measure[:1])

    assert wellformednesstools.is_well_formed_component(measure)
    assert measure.lilypond_format == "{\n\t\\time 7/16\n\tc'16\n\tc'8\n\tc'8\n\tc'8\n}"


def test_Measure___delitem___05():
    '''Trim non-power-of-two measure.

    Automatically update time signature.
    '''

    measure = Measure((4, 9), "c'8 d'8 e'8 f'8")
    measure.automatically_adjust_time_signature = True
    del(measure[:1])

    r'''
    {
        \time 3/9
        \scaleDurations #'(8 . 9) {
            d'8
            e'8
            f'8
        }
    }
    '''

    assert wellformednesstools.is_well_formed_component(measure)
    assert measure.lilypond_format == "{\n\t\\time 3/9\n\t\\scaleDurations #'(8 . 9) {\n\t\td'8\n\t\te'8\n\t\tf'8\n\t}\n}"


def test_Measure___delitem___06():
    '''Trim non-power-of-two measure, with denominator change.

    Automatically update time signature.
    '''

    measure = Measure((3, 9), "c'16 d'16 e'8 f'8")
    measure.automatically_adjust_time_signature = True

    r'''
    {
        \time 3/9
        \scaleDurations #'(8 . 9) {
            c'16
            d'16
            e'8
            f'8
        }
    }
    '''

    del(measure[:1])

    r'''
    {
        \time 5/18
        \scaleDurations #'(8 . 9) {
            d'16
            e'8
            f'8
        }
    }
    '''

    assert wellformednesstools.is_well_formed_component(measure)
    assert measure.lilypond_format == "{\n\t\\time 5/18\n\t\\scaleDurations #'(8 . 9) {\n\t\td'16\n\t\te'8\n\t\tf'8\n\t}\n}"


def test_Measure___delitem___07():
    '''Nonnegative indices work.
    
    Do NOT automatically update time signature.
    '''

    measure = Measure((4, 8), "c'8 c' c' c'")
    del(measure[:1])

    assert not wellformednesstools.is_well_formed_component(measure)
    assert len(measure) == 3
    assert contexttools.get_time_signature_mark_attached_to_component(measure) == (4, 8)


def test_Measure___delitem___08():
    '''Non-power-of-two measure.

    Do NOT automatically update time signature.
    '''
 
    measure = Measure((4, 9), "c'8 d' e' f'")
    del(measure[:1])

    assert not wellformednesstools.is_well_formed_component(measure)
    assert len(measure) == 3
    assert contexttools.get_time_signature_mark_attached_to_component(measure) == (4, 9)



def transpose_chromatic_pitch_class_number_to_chromatic_pitch_number_neighbor(
    chromatic_pitch_number, chromatic_pitch_class_number):
    '''.. versionadded:: 1.1

    Transpose `chromatic_pitch_class_number` by octaves to nearest neighbor
    of `chromatic_pitch_number`::

        >>> pitchtools.transpose_chromatic_pitch_class_number_to_chromatic_pitch_number_neighbor(
        ...     12, 4)
        16

    Resulting chromatic pitch number must be within one tritone of `chromatic_pitch_number`.

    Return chromatic pitch number.
    '''

    target_pc = chromatic_pitch_number % 12

    down = (target_pc - chromatic_pitch_class_number) % 12
    up = (chromatic_pitch_class_number - target_pc) % 12

    if up < down:
        return chromatic_pitch_number + up
    else:
        return chromatic_pitch_number - down



from abjad import *


def test_RhythmTreeContainer___len___01():

    leaf_a = rhythmtreetools.RhythmTreeLeaf(duration=3)
    leaf_b = rhythmtreetools.RhythmTreeLeaf(duration=3)
    leaf_c = rhythmtreetools.RhythmTreeLeaf(duration=2)
    subcontainer = rhythmtreetools.RhythmTreeContainer(
        duration=2, children=[leaf_b, leaf_c])
    leaf_d = rhythmtreetools.RhythmTreeLeaf(duration=1)
    
    container = rhythmtreetools.RhythmTreeContainer(
        duration=1, children=[leaf_a, subcontainer, leaf_d])

    assert len(container) == 3



from abjad.tools import contexttools
from abjad.tools import durationtools
from abjad.tools.quantizationtools.QSchemaItem import QSchemaItem


class MeasurewiseQSchemaItem(QSchemaItem):
    '''`MeasurewiseQSchemaItem` represents a change of state in the timeline of a metered
    quantization process.

    ::

        >>> from abjad.tools import quantizationtools
        >>> quantizationtools.MeasurewiseQSchemaItem()
        MeasurewiseQSchemaItem()

    Define a change in tempo:

    ::

        >>> quantizationtools.MeasurewiseQSchemaItem(tempo=((1, 4), 60))
        MeasurewiseQSchemaItem(
            tempo=contexttools.TempoMark(
                durationtools.Duration(1, 4),
                60
                ),
            )

    Define a change in time signature:

    ::

        >>> quantizationtools.MeasurewiseQSchemaItem(time_signature=((6, 8)))
        MeasurewiseQSchemaItem(
            time_signature=contexttools.TimeSignatureMark(
                (6, 8)
                ),
            )

    Test for beatspan, given a defined time signature:

    ::

        >>> _.beatspan
        Duration(1, 8)

    `MeasurewiseQSchemaItem` is immutable.

    Return `MeasurewiseQSchemaItem` instance.
    '''

    ### CLASS ATTRIBUTES ###

    __slots__ = ()
    _fields = ('search_tree', 'tempo', 'time_signature', 'use_full_measure')

    ### INITIALIZER ###

    def __new__(klass, search_tree=None, tempo=None, time_signature=None, use_full_measure=None):
        from abjad.tools import quantizationtools

        if search_tree is not None:
            assert isinstance(search_tree, quantizationtools.SearchTree)

        if tempo is not None:
            tempo = contexttools.TempoMark(tempo)
            assert not tempo.is_imprecise

        if time_signature is not None:
            time_signature = contexttools.TimeSignatureMark(time_signature)

        if use_full_measure is not None:
            use_full_measure = bool(use_full_measure)

        return tuple.__new__(klass, (search_tree, tempo, time_signature, use_full_measure))

    ### READ-ONLY PUBLIC PROPERTIES ###

    @property
    def beatspan(self):
        '''The beatspan duration, if a time signature was defined.

        Return `Duration` or `None`.
        '''
        if self.time_signature is not None:
            if self.use_full_measure:
                return self.time_signature.duration
            else:
                return durationtools.Duration(1, self.time_signature.denominator)
        return None

    @property
    def search_tree(self):
        '''The optionally defined search tree.

        Return `OldSearchTree` or `None`.
        '''
        return self[0]

    @property
    def tempo(self):
        '''The optionally defined `TempoMark`.

        Return `TempoMark` or `None`.
        '''
        return self[1]

    @property
    def time_signature(self):
        '''The optionally defined TimeSignatureMark.

        Return `TimeSignatureMark` or None.
        '''
        return self[2]

    @property
    def use_full_measure(self):
        '''If True, use the full measure as the beatspan.

        Return bool or None.
        '''
        return self[3]



import py
from abjad import *
from abjad.tools.mathtools import NonreducedFraction


def test_durationtools_group_nonreduced_fractions_by_implied_prolation_01():
    assert py.test.raises(
        AssertionError, 'durationtools.group_nonreduced_fractions_by_implied_prolation([])')


def test_durationtools_group_nonreduced_fractions_by_implied_prolation_02():
    t = durationtools.group_nonreduced_fractions_by_implied_prolation([(1, 4)])
    assert t == [[NonreducedFraction(1, 4)]]


def test_durationtools_group_nonreduced_fractions_by_implied_prolation_03():
    t = durationtools.group_nonreduced_fractions_by_implied_prolation([(1, 4), (1, 4), (1, 8)])
    assert t == [[NonreducedFraction(1, 4), NonreducedFraction(1, 4), NonreducedFraction(1, 8)]]


def test_durationtools_group_nonreduced_fractions_by_implied_prolation_04():
    t = durationtools.group_nonreduced_fractions_by_implied_prolation([(1, 4), (1, 3), (1, 8)])
    assert t == [[NonreducedFraction(1, 4)], [NonreducedFraction(1, 3)], [NonreducedFraction(1, 8)]]


def test_durationtools_group_nonreduced_fractions_by_implied_prolation_05():
    t = durationtools.group_nonreduced_fractions_by_implied_prolation([(1, 4), (1, 2), (1, 3)])
    assert t == [[NonreducedFraction(1, 4), NonreducedFraction(1, 2)], [NonreducedFraction(1, 3)]]


def test_durationtools_group_nonreduced_fractions_by_implied_prolation_06():
    t = durationtools.group_nonreduced_fractions_by_implied_prolation([(1, 4), (1, 2), (1, 3), (1, 6),
        (1, 5)])
    assert t == [[NonreducedFraction(1, 4), NonreducedFraction(1, 2)], [NonreducedFraction(1, 3), NonreducedFraction(1, 6)], [NonreducedFraction(1, 5)]]


def test_durationtools_group_nonreduced_fractions_by_implied_prolation_07():
    t = durationtools.group_nonreduced_fractions_by_implied_prolation([(1, 24), (2, 24), (3, 24),
        (4, 24), (5, 24), (6, 24)])
    assert t == [[NonreducedFraction(1, 24), NonreducedFraction(2, 24), NonreducedFraction(3, 24), NonreducedFraction(4, 24), NonreducedFraction(5, 24), NonreducedFraction(6, 24)]]
